/*
    -- MAGMA (version 1.4.0) --
       Univ. of Tennessee, Knoxville
       Univ. of California, Berkeley
       Univ. of Colorado, Denver
       @date July 2015

       @author Mark Gates
       @generated from zprint.cpp normal z -> c, Sun Jul 12 17:46:25 2015

*/
#include "common_magma.h"

#define A(i,j) (A + (i) + (j)*lda)

// -------------------------
// Prints a matrix that is on the CPU host.
extern "C"
void magma_cprint( magma_int_t m, magma_int_t n, magmaFloatComplex *A, magma_int_t lda )
{
    magmaFloatComplex c_zero = MAGMA_C_ZERO;
    
    printf( "[\n" );
    for( magma_int_t i = 0; i < m; ++i ) {
        for( magma_int_t j = 0; j < n; ++j ) {
            if ( MAGMA_C_EQUAL( *A(i,j), c_zero )) {
                printf( "   0.    " );
            }
            else {
                printf( " %8.4f", MAGMA_C_REAL( *A(i,j) ));
            }
        }
        printf( "\n" );
    }
    printf( "];\n" );
}

// -------------------------
// Prints a matrix that is on the GPU device.
// Internally allocates memory on host, copies it to the host, prints it,
// and de-allocates host memory.
extern "C"
void magma_cprint_gpu( magma_int_t m, magma_int_t n, magmaFloatComplex_ptr dA, magma_int_t ldda )
{
    magma_int_t lda = m;
    magmaFloatComplex* A = (magmaFloatComplex*) malloc( lda*n*sizeof(magmaFloatComplex) );
    magma_cgetmatrix( m, n, dA, ldda, 0, A, lda, 0, NULL );
    
    magma_cprint( m, n, A, lda );
    
    free( A );
}
