/*
 *   -- clMAGMA (version 1.4.0) --
 *      Univ. of Tennessee, Knoxville
 *      Univ. of California, Berkeley
 *      Univ. of Colorado, Denver
 *      @date July 2015
 *
 * @author Mark Gates
 * @generated from magma_z.h normal z -> d, Sun Jul 12 17:46:24 2015
 */

#ifndef MAGMA_D_H
#define MAGMA_D_H

#include "magma_types.h"
// To Do - fix circular dependency for MAGMA MIC sparse 
// #include "../sparse-iter/include/magmasparse_types.h"

#define PRECISION_d

#ifdef __cplusplus
extern "C" {
#endif

/* ////////////////////////////////////////////////////////////////////////////
   -- MAGMA Auxiliary functions to get the NB used
*/
magma_int_t magma_get_dlaex3_m_nb();       // defined in dlaex3_m.cpp

magma_int_t magma_get_dpotrf_nb( magma_int_t m );
magma_int_t magma_get_dgetrf_nb( magma_int_t m );
magma_int_t magma_get_dgetri_nb( magma_int_t m );
magma_int_t magma_get_dgeqp3_nb( magma_int_t m );
magma_int_t magma_get_dgeqrf_nb( magma_int_t m );
magma_int_t magma_get_dgeqlf_nb( magma_int_t m );
magma_int_t magma_get_dgehrd_nb( magma_int_t m );
magma_int_t magma_get_dsytrd_nb( magma_int_t m );
magma_int_t magma_get_dsytrf_nb( magma_int_t m );
magma_int_t magma_get_dsytrf_nopiv_nb( magma_int_t m );
magma_int_t magma_get_dgelqf_nb( magma_int_t m );
magma_int_t magma_get_dgebrd_nb( magma_int_t m );
magma_int_t magma_get_dsygst_nb( magma_int_t m );
magma_int_t magma_get_dgesvd_nb( magma_int_t m );
magma_int_t magma_get_dsygst_nb_m( magma_int_t m );
magma_int_t magma_get_dbulge_nb( magma_int_t m, magma_int_t nbthreads );
magma_int_t magma_get_dbulge_nb_mgpu( magma_int_t m );
magma_int_t magma_dbulge_get_Vblksiz( magma_int_t m, magma_int_t nb, magma_int_t nbthreads );
magma_int_t magma_get_dbulge_gcperf();

/* ////////////////////////////////////////////////////////////////////
 magma symv for cpu
 */

void magmablas_dsymv_work_cpu(magma_uplo_t uplo, magma_int_t *pN, double *palpha, double *A, magma_int_t *plda, double *X, magma_int_t *pincx, double *pbeta, double *Y, magma_int_t *pincy, double *y);

void magmablas_dsymv_cpu(magma_uplo_t uplo, magma_int_t *pN, double *palpha, double *A, magma_int_t *plda, double *X, magma_int_t *pincx, double *pbeta, double *Y, magma_int_t *pincy);

/* ////////////////////////////////////////////////////////////////////////////
   -- MAGMA function definitions / Data on CPU
*/
magma_int_t
magma_dgetrf(magma_int_t m, magma_int_t n,
             double *a, magma_int_t lda,
             magma_int_t *ipiv, magma_int_t *info,
             magma_queue_t queue );

magma_int_t
magma_dgetf2_nopiv(magma_int_t m, magma_int_t n,
             double *A, magma_int_t lda,
             magma_int_t *info);

magma_int_t
magma_dgetrf_nopiv(magma_int_t m, magma_int_t n,
    double *A, magma_int_t lda,
    magma_int_t *info);

magma_int_t
magma_dgetrf3(magma_int_t m, magma_int_t n,
             double *a, magma_int_t lda, double *da, magma_int_t ldda,
             magma_int_t *ipiv, magma_int_t *d_ipiv, magma_int_t *info,
             magma_queue_t queue );


magma_int_t
magma_dpotrf(
        magma_uplo_t uplo,
        magma_int_t n,
        magmaDouble_ptr hA, magma_int_t lda,
        magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dpotrf3(
        magma_uplo_t uplo,
        magma_int_t n,
        magmaDouble_ptr hA, magma_int_t lda, magmaDouble_ptr dA, magma_int_t ldda,
        magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dgeqrf(magma_int_t m, magma_int_t n,
             double *hA, magma_int_t lda,
             double *tau, double *hwork,
             magma_int_t lwork, magma_int_t *info,
             magma_queue_t queue );

magma_int_t
magma_dgeqrf3(magma_int_t m, magma_int_t n,
             double *hA, magma_int_t lda, double *dA, magma_int_t ldda,
             double *tau, double *hwork,
             magma_int_t lwork, magma_int_t *info,
             magma_queue_t queue );

magma_int_t
magma_dgesdd(magma_vec_t jobz, magma_int_t m, magma_int_t n,
             double *A, magma_int_t lda,
             double *s,
             double *U, magma_int_t ldu,
             double *VT, magma_int_t ldvt,
             double *work, magma_int_t lwork,
             #if defined(PRECISION_z) || defined(PRECISION_c)
             double *rwork,
             #endif
             magma_int_t *iwork,
             magma_queue_t queue, magma_int_t *info);

magma_int_t
magma_dgetrf2(magma_int_t m, magma_int_t n,
              double *a, magma_int_t lda,
              magma_int_t *ipiv,
              magmaDouble_ptr dA, magma_int_t ldda, magma_int_t *d_ipiv,
              magma_int_t *info, magma_queue_t queue );

#if defined(PRECISION_z) || defined(PRECISION_c)
magma_int_t
magma_dgeev(  magma_vec_t jobvl, magma_vec_t jobvr, magma_int_t n,
              double *a, magma_int_t lda,
              double *w,
              double *vl, magma_int_t ldvl,
              double *vr, magma_int_t ldvr,
              double *work, magma_int_t lwork,
              double *rwork, magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dgesvd( magma_vec_t jobu, magma_vec_t jobvt, magma_int_t m, magma_int_t n,
              double *a,    magma_int_t lda, double *s,
              double *u,    magma_int_t ldu,
              double *vt,   magma_int_t ldvt,
              double *work, magma_int_t lwork,
              double *rwork, magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_dsyevd( magma_vec_t jobz, magma_uplo_t uplo, magma_int_t n,
              double *a, magma_int_t lda, double *w,
              double *work, magma_int_t lwork,
              double *rwork, magma_int_t lrwork,
              magma_int_t *iwork, magma_int_t liwork, magma_int_t *info,
              magma_queue_t queue );

magma_int_t
magma_dstedx( magma_range_t range, magma_int_t n, double vl, double vu,
              magma_int_t il, magma_int_t iu, double *D, double *E,
              double *Z, magma_int_t ldz,
              double *rwork, magma_int_t ldrwork, magma_int_t *iwork,
              magma_int_t liwork, double *dwork, magma_int_t *info,
              magma_queue_t queue );

#else /* not (defined(PRECISION_z) || defined(PRECISION_c)) */

magma_int_t
magma_dgeev(  magma_vec_t jobvl, magma_vec_t jobvr, magma_int_t n,
              double *a,    magma_int_t lda,
              double *wr, double *wi,
              double *vl,   magma_int_t ldvl,
              double *vr,   magma_int_t ldvr,
              double *work, magma_int_t lwork,
              magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dgesvd( magma_vec_t jobu, magma_vec_t jobvt, magma_int_t m, magma_int_t n,
              double *a,    magma_int_t lda, double *s,
              double *u,    magma_int_t ldu,
              double *vt,   magma_int_t ldvt,
              double *work, magma_int_t lwork,
              magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_dsyevd( magma_vec_t jobz, magma_uplo_t uplo, magma_int_t n,
              double *a, magma_int_t lda, double *w,
              double *work, magma_int_t lwork,
              magma_int_t *iwork, magma_int_t liwork, magma_int_t *info,
              magma_queue_t queue);

magma_int_t
magma_dstedx( magma_range_t range, magma_int_t n, double vl, double vu,
              magma_int_t il, magma_int_t iu, double *d, double *e,
              double *z, magma_int_t ldz,
              double *work, magma_int_t lwork,
              magma_int_t *iwork, magma_int_t liwork,
              double *dwork, magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dlaex0( magma_int_t n, double *d, double *e, double *q, magma_int_t ldq,
              double *work, magma_int_t *iwork, double *dwork,
              magma_range_t range, double vl, double vu,
              magma_int_t il, magma_int_t iu, magma_int_t *info,
              magma_queue_t queue);

magma_int_t
magma_dlaex1( magma_int_t n, double *d, double *q, magma_int_t ldq,
              magma_int_t *indxq, double rho, magma_int_t cutpnt,
              double *work, magma_int_t *iwork, double *dwork,
              magma_range_t range, double vl, double vu,
              magma_int_t il, magma_int_t iu, magma_int_t *info,
              magma_queue_t queue);

magma_int_t
magma_dlaex3( magma_int_t k, magma_int_t n, magma_int_t n1, double *d,
              double *q, magma_int_t ldq, double rho,
              double *dlamda, double *q2, magma_int_t *indx,
              magma_int_t *ctot, double *w, double *s, magma_int_t *indxq,
              double *dwork,
              magma_range_t range, double vl, double vu, magma_int_t il, magma_int_t iu,
              magma_int_t *info, magma_queue_t queue );

#endif /* not (defined(PRECISION_z) || defined(PRECISION_c)) */


/* ////////////////////////////////////////////////////////////////////////////
 -- MAGMA function definitions / Data on MIC
*/

magma_int_t
magma_dgebrd(magma_int_t m, magma_int_t n,
             double *a, magma_int_t lda, double *d, double *e,
             double *tauq, double *taup,
             double *work, magma_int_t lwork,
             magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dgeqrf2_mic(
        magma_int_t m, magma_int_t n,
        magmaDouble_ptr dA, size_t dA_offset, magma_int_t ldda,
        double *tau, magma_int_t *info, magma_queue_t queue);

magma_int_t 
magma_dgerbt_mic(magma_bool_t gen, magma_int_t n, magma_int_t nrhs, 
        double *dA, magma_int_t ldda, 
        double *dB, magma_int_t lddb, 
        double *U, double *V, 
        magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dgetrf_mic(
        magma_int_t m, magma_int_t n,
        magmaDouble_ptr dA, size_t dA_offset, magma_int_t ldda,
        magma_int_t *ipiv, magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dgetrf_nopiv_mic(
        magma_int_t m, magma_int_t n,
        magmaDouble_ptr dA, size_t dA_offset, magma_int_t ldda,
        magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dgetrf_mmic(
        magma_int_t num_mics, magma_int_t m, magma_int_t n,
        magmaDouble_ptr *dA, magma_int_t ldda,
        magma_int_t *ipiv, magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dgeqrf2_mmic(
        magma_int_t num_mics, magma_int_t m, magma_int_t n,
        magmaDouble_ptr *dlA, magma_int_t ldda,
        double *tau, magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dsytrf_nopiv_mic(
        magma_uplo_t uplo, magma_int_t n,
        magmaDouble_ptr dA, size_t dA_offset, magma_int_t ldda,
        magma_queue_t queue,
        magma_int_t *info);

magma_int_t
magma_dlarfb_mic(
        magma_side_t side, magma_trans_t trans, magma_direct_t direct, magma_storev_t storev,
        magma_int_t m, magma_int_t n, magma_int_t k,
        magmaDouble_ptr dV, size_t dV_offset, magma_int_t ldv,
        magmaDouble_ptr dT, size_t dT_offset, magma_int_t ldt,
        magmaDouble_ptr dC, size_t dC_offset, magma_int_t ldc,
        magmaDouble_ptr dwork, size_t dwork_offset, magma_int_t ldwork,
        magma_queue_t queue);

magma_int_t
magma_dlarfb_mic_gemm(
        magma_side_t side, magma_trans_t trans, magma_direct_t direct, magma_storev_t storev,
        magma_int_t m, magma_int_t n, magma_int_t k,
        magmaDouble_ptr dV, size_t dV_offset, magma_int_t ldv,
        magmaDouble_ptr dT, size_t dT_offset, magma_int_t ldt,
        magmaDouble_ptr dC, size_t dC_offset, magma_int_t ldc,
        magmaDouble_ptr dwork, size_t dwork_offset, magma_int_t ldwork,
        magmaDouble_ptr dworkvt, size_t dworkvt_offset, magma_int_t ldworkvt,
        magma_queue_t queue);

magma_int_t
magma_dpotrf_mic(
        magma_uplo_t uplo,
        magma_int_t n,
        magmaDouble_ptr dA, size_t dA_offset, magma_int_t ldda,
        magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dpotrf_mmic(
        magma_int_t num_mics,
        magma_uplo_t uplo,
        magma_int_t n,
        magmaDouble_ptr *dA, magma_int_t ldda,
        magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dpotrs_mic(
         magma_uplo_t uplo, magma_int_t n, magma_int_t nrhs,
         magmaDouble_ptr dA, size_t dA_offset, magma_int_t ldda,
         magmaDouble_ptr dB, size_t dB_offset, magma_int_t lddb,
         magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_dposv(
                magma_uplo_t uplo, magma_int_t n, magma_int_t nrhs,
                magmaDouble_ptr A, magma_int_t ldda,
                magmaDouble_ptr B, magma_int_t lddb,
                magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_dposv_mic(
                magma_uplo_t uplo, magma_int_t n, magma_int_t nrhs,
                magmaDouble_ptr dA, size_t dA_offset, magma_int_t ldda,
                magmaDouble_ptr dB, size_t dB_offset, magma_int_t lddb,
                magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_dgetri_mic( magma_int_t n,
                double *dA, magma_int_t ldda, magma_int_t *ipiv,
                double *dwork, magma_int_t lwork, magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dtrtri_mic( magma_uplo_t uplo,  magma_diag_t diag, magma_int_t n,
                double *dA, magma_int_t ldda, magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dgetrs_mic(magma_trans_t trans, magma_int_t n, magma_int_t nrhs,
                 magmaDouble_ptr dA, size_t dA_offset, magma_int_t ldda,
                 magma_int_t *ipiv,
                 magmaDouble_ptr dB, size_t dB_offset, magma_int_t lddb,
                 magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dgetrs_nopiv_mic(magma_trans_t trans, magma_int_t n, magma_int_t nrhs,
                 magmaDouble_ptr dA, size_t dA_offset, magma_int_t ldda,
                 magmaDouble_ptr dB, size_t dB_offset, magma_int_t lddb,
                 magma_int_t *info, magma_queue_t queue);
magma_int_t
magma_dgesv_rbt( magma_bool_t ref, magma_int_t n, magma_int_t nrhs,
                 double *A, magma_int_t lda, 
                 double *B, magma_int_t ldb, 
                 magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dgesv(     magma_int_t n, magma_int_t nrhs,
                 double *A, magma_int_t lda, magma_int_t *ipiv,
                 double *B, magma_int_t ldb, magma_int_t *info, magma_queue_t queue);
magma_int_t
magma_dgesv_mic( magma_int_t n, magma_int_t nrhs,
                 magmaDouble_ptr dA, size_t dA_offset, magma_int_t ldda,
                 magma_int_t *ipiv,
                 magmaDouble_ptr dB, size_t dB_offset, magma_int_t lddb,
                 magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_dgesv_nopiv_mic( magma_int_t n, magma_int_t nrhs,
                 magmaDouble_ptr dA, size_t dA_offset, magma_int_t ldda,
                 magmaDouble_ptr dB, size_t dB_offset, magma_int_t lddb,
                 magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_dormqr_mic(magma_side_t side, magma_trans_t trans,
                 magma_int_t m, magma_int_t n, magma_int_t k,
                 magmaDouble_ptr dA, size_t dA_offset, magma_int_t ldda,
                 double *tau,
                 magmaDouble_ptr dC, size_t dC_offset, magma_int_t lddc,
                 double *hwork, magma_int_t lwork,
                 magmaDouble_ptr dT, size_t dT_offset, magma_int_t nb,
                 magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dgeqrs_mic(magma_int_t m, magma_int_t n, magma_int_t nrhs,
                 magmaDouble_ptr dA, size_t dA_offset, magma_int_t ldda,
                 double *tau,   magmaDouble_ptr dT, size_t dT_offset,
                 magmaDouble_ptr dB, size_t dB_offset, magma_int_t lddb,
                 double *hwork, magma_int_t lwork,
                 magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dgeqrf_mic( magma_int_t m, magma_int_t n,
                  magmaDouble_ptr dA, size_t dA_offset,  magma_int_t ldda,
                  double *tau, magmaDouble_ptr dT, size_t dT_offset,
                  magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dgels_mic( magma_trans_t trans, magma_int_t m, magma_int_t n, magma_int_t nrhs,
                 magmaDouble_ptr dA, size_t dA_offset,  magma_int_t ldda,
                 magmaDouble_ptr dB, size_t dB_offset,  magma_int_t lddb,
                 double *hwork, magma_int_t lwork,
                 magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_dgehrd(   magma_int_t n, magma_int_t ilo, magma_int_t ihi,
                double *a, magma_int_t lda,
                double *tau,
                double *work, magma_int_t lwork,
                magmaDouble_ptr dT, size_t dT_offset,
                magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_dlabrd_mic( magma_int_t m, magma_int_t n, magma_int_t nb,
                  double *a, magma_int_t lda,
                  magmaDouble_ptr da, size_t da_offset, magma_int_t ldda,
                  double *d, double *e, double *tauq, double *taup,
                  double *x, magma_int_t ldx,
                  magmaDouble_ptr dx, size_t dx_offset, magma_int_t lddx,
                  double *y, magma_int_t ldy,
                  magmaDouble_ptr dy, size_t dy_offset, magma_int_t lddy,
                  magma_int_t pstep, magma_queue_t queue );

magma_int_t
magma_dlasyf_mic(
		 magma_uplo_t uplo, magma_int_t n, magma_int_t nb, magma_int_t *kb,
		 double *hA, magma_int_t lda,
		 magmaDouble_ptr dA, size_t dA_offset, magma_int_t ldda,
		 magma_int_t *ipiv,
		 magmaDouble_ptr dW, size_t dW_offset, magma_int_t lddw,
		 magma_int_t *info,
		 magma_queue_t queue);

magma_int_t
dsytrf_nopiv_cpu(
                 magma_uplo_t uplo, magma_int_t n, magma_int_t ib,
                 double *A, magma_int_t lda,
                 magma_int_t *info);

magma_int_t
magma_dlahr2(   magma_int_t n, magma_int_t k, magma_int_t nb,
                magmaDouble_ptr da, size_t da_offset, magmaDouble_ptr dv, size_t dv_offset,
                double *a, magma_int_t lda,
                double *tau, double *t, magma_int_t ldt,
                double *y, magma_int_t ldy,
                magma_queue_t queue     );

magma_int_t
magma_dlahru(   magma_int_t n, magma_int_t ihi, magma_int_t k, magma_int_t nb,
                double *a, magma_int_t lda,
                magmaDouble_ptr d_a, size_t d_a_offset, magmaDouble_ptr y, size_t y_offset,
                magmaDouble_ptr v, size_t v_offset, magmaDouble_ptr d_t, size_t dt_offset,
                magmaDouble_ptr d_work, size_t d_work_offset, magma_queue_t queue );

magma_int_t
magma_dorghr(   magma_int_t n, magma_int_t ilo, magma_int_t ihi,
                double *a, magma_int_t lda,
                double *tau,
                magmaDouble_ptr dT, size_t dT_offset, magma_int_t nb,
                magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_dorgqr(   magma_int_t m, magma_int_t n, magma_int_t k,
                double *a, magma_int_t lda,
                double *tau, magmaDouble_ptr dT, size_t dT_offset,
                magma_int_t nb, magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_dorgqr2(  magma_int_t m, magma_int_t n, magma_int_t k,
                double *A, magma_int_t lda,
                double *tau,
                magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dormbr(
                magma_vect_t vect, magma_side_t side, magma_trans_t trans,
                magma_int_t m, magma_int_t n, magma_int_t k,
                double *A, magma_int_t lda,
                double *tau,
                double *C, magma_int_t ldc,
                double *work, magma_int_t lwork,
                magma_queue_t queue, magma_int_t *info);

magma_int_t
magma_dormlq(
                magma_side_t side, magma_trans_t trans,
                magma_int_t m, magma_int_t n, magma_int_t k,
                double *A, magma_int_t lda,
                double *tau,
                double *C, magma_int_t ldc,
                double *work, magma_int_t lwork,
                magma_queue_t queue, magma_int_t *info);

magma_int_t
magma_dormql(   magma_side_t side, magma_trans_t trans,
                magma_int_t m, magma_int_t n, magma_int_t k,
                double *a, magma_int_t lda,
                double *tau,
                double *c, magma_int_t ldc,
                double *work, magma_int_t lwork,
                magma_int_t *info, magma_queue_t queue );

magma_int_t
magma_dormqr(   magma_side_t side, magma_trans_t trans,
                magma_int_t m, magma_int_t n, magma_int_t k,
                double *a, magma_int_t lda, double *tau,
                double *c, magma_int_t ldc,
                double *work, magma_int_t lwork,
                magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dormtr(   magma_side_t side, magma_uplo_t uplo, magma_trans_t trans,
                magma_int_t m, magma_int_t n,
                double *a,    magma_int_t lda,
                double *tau,
                double *c,    magma_int_t ldc,
                double *work, magma_int_t lwork,
                magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dlatrd(magma_uplo_t uplo, magma_int_t n, magma_int_t nb,
             double *a,  magma_int_t lda,
             double *e, double *tau,
             double *w,  magma_int_t ldw,
             double *da, size_t da_offset, magma_int_t ldda,
             double *dw, size_t dw_offset, magma_int_t lddw, 
             magma_int_t pstep, double *dpartialY, magma_queue_t queue);

magma_int_t
magma_dsytrd(  magma_uplo_t uplo, magma_int_t n,
               double *a, magma_int_t lda,
               double *d, double *e, double *tau,
               double *work, magma_int_t lwork,
               magma_int_t *info, magma_queue_t queue);

magma_int_t
magma_dsytrf(  magma_uplo_t uplo, magma_int_t n,
	       double *A, magma_int_t lda,
	       magmaDouble_ptr da, magma_int_t ldda,
	       magma_int_t *ipiv, magma_int_t *d_ipiv,
	       magma_queue_t queue, magma_int_t *info );

magma_int_t
magma_dsytrf_nopiv(
               magma_uplo_t uplo, magma_int_t n,
               double *A, magma_int_t lda,
               magma_queue_t queue,
               magma_int_t *info);


//##############################################################################
// MAGMA Sparse-Iter
//

magma_int_t
magma_djacobidiagscale(
    magma_int_t n,
    magma_d_vector b,
    magma_d_vector d,
    magma_d_vector *c,
    magma_queue_t queue );

/* ////////////////////////////////////////////////////////////////////////////
 -- MAGMA utility function definitions
*/

void magma_dprint    ( magma_int_t m, magma_int_t n, double     *A, magma_int_t lda  );
void magma_dprint_gpu( magma_int_t m, magma_int_t n, magmaDouble_ptr dA, magma_int_t ldda );

void dpanel_to_q(magma_uplo_t uplo, magma_int_t ib, double *A, magma_int_t lda, double *work);
void dq_to_panel(magma_uplo_t uplo, magma_int_t ib, double *A, magma_int_t lda, double *work);

#ifdef __cplusplus
}
#endif

#undef PRECISION_d
#endif /* MAGMA_D_H */
