/*
 *   -- micMAGMA (version 1.4.0) --
 *      Univ. of Tennessee, Knoxville
 *      Univ. of California, Berkeley
 *      Univ. of Colorado, Denver
 *      @date July 2015
 *
 *   @author Azzam Haidar
 *   @author Khairul Kabir
 *   @author Stan Tomov
 *   @author Hartwig Anzt
 *
 *   @precisions normal z -> s d c
 */


#ifndef MAGMA_Z_BINDINGS
#define MAGMA_Z_BINDINGS
#define PRECISION_z

typedef struct {
    magma_trans_t transa;
    magma_int_t m;
    magmaDoubleComplex_const_ptr a;
    magmaIndex_const_ptr ia;
    magmaIndex_const_ptr ja;
    magmaDoubleComplex_const_ptr x;
    magmaDoubleComplex_ptr y;
} magma_mic_zcsrgemv_param;

typedef struct {
    magma_int_t m;
    magmaDoubleComplex_const_ptr a;
    magmaIndex_const_ptr ia;
    magmaIndex_const_ptr ja;
    magmaDoubleComplex_ptr x;
    magmaDoubleComplex_const_ptr b;

    magmaDoubleComplex_ptr dwork;
} magma_mic_zdebug_param;

typedef struct {
    magma_int_t m;
    magmaDoubleComplex_const_ptr a;
    magmaIndex_const_ptr ia;
    magmaIndex_const_ptr ja;
    magmaDoubleComplex_ptr x;
    magmaDoubleComplex_const_ptr b;

    magma_z_solver_par *solver_par;
    magmaDoubleComplex_ptr dwork;
} magma_mic_zbicgstab_param;

typedef struct {
    magma_int_t m;
    magmaDoubleComplex_const_ptr a;
    magmaIndex_const_ptr ia;
    magmaIndex_const_ptr ja;
    magmaDoubleComplex_ptr x;
    magmaDoubleComplex_const_ptr b;

    magma_z_solver_par *solver_par;
    magmaDoubleComplex_ptr dwork;
} magma_mic_zcg_param;

typedef struct {
    magma_int_t m;
    magmaDoubleComplex_const_ptr a;
    magmaIndex_const_ptr ia;
    magmaIndex_const_ptr ja;
    magmaDoubleComplex_ptr x;
    magmaDoubleComplex_const_ptr b;

    magma_z_solver_par *solver_par;
    magmaDoubleComplex_ptr dwork;
} magma_mic_zgmres_param;

typedef struct {
    magma_trans_t transa;
    magma_int_t request;
    magma_int_t sort;
    magma_int_t  m;
    magma_int_t  n;
    magma_int_t  k;
    magmaDoubleComplex alpha;
    magmaDoubleComplex_const_ptr a;
    magmaIndex_const_ptr ia;
    magmaIndex_const_ptr ja;
    magmaDoubleComplex_const_ptr b;
    magmaIndex_const_ptr ib;
    magmaIndex_const_ptr jb;
    magmaDoubleComplex_ptr c;
    magmaIndex_ptr ic;
    magmaIndex_ptr jc;
    magma_int_t nzmax;
    magma_int_t info;
} magma_mic_zcsrgespmm_param;

typedef struct {
    magmaDoubleComplex_ptr res;
    magma_int_t n;
    magmaDoubleComplex_const_ptr x;
    magma_int_t incx;
    magmaDoubleComplex_const_ptr y;
    magma_int_t incy;
} magma_mic_zdotc_param;

typedef struct {
    magmaDouble_ptr res;
    magma_int_t n;
    magmaDoubleComplex_const_ptr x;
    magma_int_t incx;
} magma_mic_dznrm2_param;

typedef struct {
    magma_int_t n;
    magmaDoubleComplex a;
    magmaDoubleComplex_const_ptr x;
    magma_int_t incx;
    magmaDoubleComplex_ptr y;
    magma_int_t incy;
} magma_mic_zaxpy_param;

typedef struct {
    magma_int_t n;
    magmaDoubleComplex_const_ptr x;
    magma_int_t incx;
    magmaDoubleComplex_ptr y;
    magma_int_t incy;
} magma_mic_zcopy_param;

typedef struct {
    magma_trans_t transa;
    magma_trans_t transb;
    magma_int_t m;
    magma_int_t n;
    magma_int_t k;
    magmaDoubleComplex alpha;
    magmaDoubleComplex_const_ptr a;
    magma_int_t lda;
    magmaDoubleComplex_const_ptr b;
    magma_int_t ldb;
    magmaDoubleComplex beta;
    magmaDoubleComplex_const_ptr c;
    magma_int_t ldc;
} magma_mic_zgemm_param;

typedef struct {
    magma_side_t side;
    magma_uplo_t uplo;
    magma_int_t m;
    magma_int_t n;
    magmaDoubleComplex alpha;
    magmaDoubleComplex_const_ptr a;
    magma_int_t lda;
    magmaDoubleComplex_const_ptr b;
    magma_int_t ldb;
    magmaDoubleComplex beta;
    magmaDoubleComplex_const_ptr c;
    magma_int_t ldc;
    magma_int_t agboff;
} magma_mic_zhemm_param;

typedef struct {
    magma_trans_t transa;
    magma_int_t m;
    magma_int_t n;
    magmaDoubleComplex alpha;
    magmaDoubleComplex_const_ptr a;
    magma_int_t lda;
    magmaDoubleComplex_const_ptr x;
    magma_int_t incx;
    magmaDoubleComplex beta;
    magmaDoubleComplex_const_ptr y;
    magma_int_t incy;
} magma_mic_zgemv_param;

typedef struct {
    magma_uplo_t uplo;
    magma_int_t n;
    magmaDoubleComplex alpha;
    magmaDoubleComplex_const_ptr a;
    magma_int_t lda;
    magmaDoubleComplex_const_ptr x;
    magma_int_t incx;
    magmaDoubleComplex beta;
    magmaDoubleComplex_const_ptr y;
    magma_int_t incy;
} magma_mic_zhemv_param;

typedef struct {
    magma_uplo_t uplo;
    magma_int_t n;
    magmaDoubleComplex alpha;
    magmaDoubleComplex_const_ptr a;
    magma_int_t lda;
    magmaDoubleComplex_const_ptr x;
    magma_int_t incx;
    magmaDoubleComplex beta;
    magmaDoubleComplex_const_ptr y;
    magma_int_t incy;
    magma_int_t symvshift;
    magmaDoubleComplex_const_ptr partialY;
} magma_mic_zhemv_for_trd_param;

typedef struct {
    magma_int_t n;
    magmaDoubleComplex_const_ptr d_A;
    magma_int_t lda;
    magmaDoubleComplex_const_ptr d_u;
    magmaDoubleComplex_const_ptr d_v;
} magma_mic_zprbt_param;


typedef struct {
    magma_int_t n;
    magmaDoubleComplex_const_ptr d_u;
    magmaDoubleComplex_const_ptr d_b;
} magma_mic_zprbt_mtv_param;


typedef struct {
    magma_int_t n;
    magmaDoubleComplex_const_ptr d_v;
    magmaDoubleComplex_const_ptr d_b;
} magma_mic_zprbt_mv_param;


typedef struct {
    magma_uplo_t uplo;
    magma_int_t n;
    magmaDoubleComplex alpha;
    magmaDoubleComplex_const_ptr a;
    magma_int_t lda;
    magmaDoubleComplex_const_ptr x;
    magma_int_t incx;
    magmaDoubleComplex beta;
    magmaDoubleComplex_const_ptr y;
    magma_int_t incy;
} magma_mic_zsymv2_param;



typedef struct {
    magma_uplo_t uplo;
    magma_trans_t trans;
    magma_int_t n;
    magma_int_t k;
    double alpha;
    magmaDoubleComplex_const_ptr a;
    magma_int_t lda;
    double beta;
    magmaDoubleComplex_const_ptr c;
    magma_int_t ldc;
} magma_mic_zherk_param;

typedef struct {
    magma_uplo_t uplo;
    magma_trans_t trans;
    magma_int_t n;
    magma_int_t k;
    double alpha;
    magmaDoubleComplex_const_ptr a;
    magma_int_t lda;
    magma_int_t aoff;
    double beta;
    magmaDoubleComplex_const_ptr c;
    magma_int_t ldc;
    magma_int_t offset;
    magma_int_t nb;
    magma_int_t nmic;
    magma_int_t my_id;
} magma_mic_zherk_mmic_param;

typedef struct {
    magma_uplo_t uplo;
    magma_trans_t trans;
    magma_int_t n;
    magma_int_t k;
    magmaDoubleComplex alpha;
    magmaDoubleComplex_const_ptr a;
    magma_int_t lda;
    magmaDoubleComplex_const_ptr b;
    magma_int_t ldb;
    double beta;
    magmaDoubleComplex_const_ptr c;
    magma_int_t ldc;
    magma_int_t agboff;
} magma_mic_zher2k_param;

typedef struct {
    magma_uplo_t uplo;
    magma_trans_t trans;
    magma_int_t n;
    magma_int_t k;
    magmaDoubleComplex alpha;
    magmaDoubleComplex_const_ptr a;
    magma_int_t lda;
    magma_int_t aoff;
    magmaDoubleComplex_const_ptr b;
    magma_int_t ldb;
    magma_int_t boff;
    double beta;
    magmaDoubleComplex_const_ptr c;
    magma_int_t ldc;
    magma_int_t offset;
    magma_int_t nb;
    magma_int_t nmic;
    magma_int_t my_id;
} magma_mic_zher2k_mmic_param;

typedef struct {
    magma_side_t side;
    magma_uplo_t uplo;
    magma_trans_t transa;
    magma_diag_t diag;
    magma_int_t m;
    magma_int_t n;
    magmaDoubleComplex alpha;
    magmaDoubleComplex_const_ptr a;
    magma_int_t lda;
    magmaDoubleComplex_const_ptr b;
    magma_int_t ldb;
} magma_mic_ztrsm_param;

typedef struct {
    magma_mic_ztrsm_param trsm_param;
    magma_mic_zgemm_param gemm_param;
} magma_mic_ztrsmgemm_param;

typedef struct {
  magma_uplo_t uplo;
  magma_trans_t trans;
  magma_diag_t diag;
  magma_int_t n;
  magmaDoubleComplex alpha;
  magmaDoubleComplex_const_ptr a;
  magma_int_t lda;
  magmaDoubleComplex_const_ptr dx;
  magma_int_t incx;
} magma_mic_ztrsv_param;

typedef struct {
  magma_uplo_t  uplo;
  magma_trans_t trans;
  magma_diag_t  diag;
  magma_int_t n;
  magmaDoubleComplex alpha;
  magmaDoubleComplex_const_ptr a;
  magma_int_t lda;
  magmaDoubleComplex_const_ptr dx;
  magma_int_t incx;
} magma_mic_ztrmv_param;
 
typedef struct {
    magma_uplo_t uplo;
    magma_int_t n; 
    magma_int_t nb; 
    magma_int_t *kb;
    magmaDoubleComplex *hA;
    magma_int_t lda;
    magmaDoubleComplex_ptr dA;
    magma_int_t ldda;
    magma_int_t *ipiv;
    magmaDoubleComplex_ptr dW;
    magma_int_t lddw;
} magma_mic_zlahef_param;

typedef struct {
    magma_direct_t direct;
    magma_storev_t storev;
    magma_int_t n;
    magma_int_t k;
    magmaDoubleComplex_const_ptr v;
    magma_int_t ldv;
    magmaDoubleComplex_const_ptr tau;
    magmaDoubleComplex_const_ptr t;
    magma_int_t ldt;
} magma_mic_zlarft_param;


typedef struct {
    magma_side_t   side;
    magma_trans_t  trans;
    magma_direct_t direct;
    magma_storev_t storev;
    magma_int_t m;
    magma_int_t n;
    magma_int_t k;
    magmaDoubleComplex_const_ptr v;
    magma_int_t ldv;
    magmaDoubleComplex_const_ptr t;
    magma_int_t ldt;
    magmaDoubleComplex_const_ptr c;
    magma_int_t ldc;
    magmaDoubleComplex_const_ptr work;
    magma_int_t ldwork;
} magma_mic_zlarfb_param;

typedef struct {
    magma_side_t   side;
    magma_trans_t  trans;
    magma_direct_t direct;
    magma_storev_t storev;
    magma_int_t m;
    magma_int_t n;
    magma_int_t k;
    magmaDoubleComplex_const_ptr v;
    magma_int_t ldv;
    magmaDoubleComplex_const_ptr t;
    magma_int_t ldt;
    magmaDoubleComplex_const_ptr c;
    magma_int_t ldc;
    magmaDoubleComplex_const_ptr work;
    magma_int_t ldwork;
    magmaDoubleComplex_const_ptr workvt;
    magma_int_t ldworkvt;
} magma_mic_zlarfb_gemm_param;

typedef struct {
    magma_uplo_t uplo;
    magma_int_t m;
    magma_int_t n;
    magmaDoubleComplex alpha;
    magmaDoubleComplex beta;
    magmaDoubleComplex_const_ptr dA;
    magma_int_t lda;
} magma_mic_zlaset_param;

typedef struct {
    magma_side_t side;
    magma_uplo_t uplo;
    magma_trans_t transa;
    magma_diag_t diag;
    magma_int_t m;
    magma_int_t n;
    magmaDoubleComplex alpha;
    magmaDoubleComplex_const_ptr a;
    magma_int_t lda;
    magmaDoubleComplex_const_ptr b;
    magma_int_t ldb;
} magma_mic_ztrmm_param;


typedef struct {
    magmaDoubleComplex_const_ptr odata;
    magma_int_t ldo;
    magmaDoubleComplex_const_ptr idata;
    magma_int_t ldi;
    magma_int_t m;
    magma_int_t n;
} magma_mic_ztranspose_param;


/*
typedef struct {
    magmaDoubleComplex_const_ptr d_AT;
    magma_int_t lda;
    magma_int_t *d_ipiv;
    magma_int_t nb;
    magma_int_t ind;
} magma_mic_zlaswp_param;
*/


typedef struct {
    magma_int_t n;
    magmaDoubleComplex_const_ptr d_A;
    magma_int_t lda;
    magma_int_t k1;
    magma_int_t k2;
    magma_int_t *d_ipiv;
    magma_int_t incx;
} magma_mic_zlaswp_param;


typedef struct {
    magma_int_t n;
    magmaDoubleComplex *dA1T;
    magma_int_t lda1;
    magmaDoubleComplex *dA2T;
    magma_int_t lda2;
} magma_mic_zswap_param;

typedef struct {
    magma_norm_t norm;
    magma_int_t m;
    magma_int_t n;
    magmaDoubleComplex_const_ptr dA; 
    magma_int_t ldda;
    magmaDouble_ptr dwork;
    double *ret;
} magma_mic_zlange_param;

typedef struct {
    magma_norm_t norm;
    magma_uplo_t uplo;
    magma_int_t n;
    magmaDoubleComplex_const_ptr dA;
    magma_int_t ldda;
    magmaDouble_ptr dwork;
    double *ret;
} magma_mic_zlanhe_param;

typedef struct {
    magma_uplo_t uplo;
    magma_int_t m;
    magma_int_t n;
    magmaDoubleComplex_const_ptr dA;
    magma_int_t ldda;
    magmaDoubleComplex_ptr dB;
    magma_int_t lddb;
} magma_mic_zlacpy_param;


typedef struct {
    magma_int_t n;
    magmaDoubleComplex alpha;
    magmaDoubleComplex_ptr dx;
    magma_int_t incx;
} magma_mic_zscal_param;


typedef struct {
    magma_uplo_t uplo;
    magma_int_t m;
    magma_int_t n;
    magmaDoubleComplex_ptr dA;
    magma_int_t ldda;
    magmaDoubleComplex_ptr dB;
    magma_int_t lddb;
} magma_mic_zlascl_diag_param;


typedef struct {
    magma_int_t n;
    magmaDoubleComplex_const_ptr dx; 
    magma_int_t incx;
    magma_int_t *ret;
} magma_mic_izamax_param;

typedef struct {
    magma_int_t m;
    magmaDoubleComplex *r; 
    magmaDoubleComplex *x; 
    magmaDoubleComplex *b; 
} magma_mic_zaxpycp2_param;



// SPARSE

typedef struct {
    magma_int_t m;
    magma_int_t n;
    magmaDoubleComplex_ptr d;
    magmaDoubleComplex_ptr b;
    magmaDoubleComplex_ptr c;
} magma_mic_zjacobidiagscale_param;


#endif  // MAGMA_Z_BINDINGS
