/*
 *   -- micMAGMA (version 1.4.0) --
 *      Univ. of Tennessee, Knoxville
 *      Univ. of California, Berkeley
 *      Univ. of Colorado, Denver
 *      @date July 2015
 *
 *   @author Azzam Haidar
 *   @author Khairul Kabir
 *   @author Mark Gates
 *   @author Hartwig Anzt
 *
 *   @generated from magmablas_z.cpp normal z -> s, Sun Jul 12 17:46:27 2015
 */

#include <stdio.h>
#include <stdlib.h>

#include "magmasparse.h"
#include "magma.h"

#define PRECISION_s

#ifdef HAVE_MIC

#include "magma_mic_bindings.h"
// ========================================
// copying vectors
extern "C" magma_int_t
magma_ssetvector(
    magma_int_t n,
    float const* hx_src, magma_int_t incx,
    float*       dy_dst, magma_int_t incy,
    magma_queue_t queue )
{
    magma_event_t event;
    magma_mic_synchronize_kernels();
    magma_mic_memcpy2D_async(
        dy_dst, incy*sizeof(float),
        hx_src, incx*sizeof(float),
        sizeof(float), n,
        WRITETO_DEV, queue, &event );
    magma_mic_fence_wait( event );
    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_sgetvector(
   magma_int_t n,
   float const* dx_src, magma_int_t incx,
   float*       hy_dst, magma_int_t incy,
   magma_queue_t queue )
{
    magma_event_t event;
    magma_mic_synchronize_kernels();
    magma_mic_memcpy2D_async(
        hy_dst, incy*sizeof(float),
        dx_src, incx*sizeof(float),
        sizeof(float), n,
        READFROM_DEV, queue, &event );
    magma_mic_fence_wait( event );
    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_ssetvector_async(
    magma_int_t n,
    float const* hx_src, magma_int_t incx,
    float*       dy_dst, magma_int_t incy,
    magma_queue_t queue, magma_event_t *event )
{
    magma_mic_memcpy2D_async(
        dy_dst, incy*sizeof(float),
        hx_src, incx*sizeof(float),
        sizeof(float), n,
        WRITETO_DEV, queue, event );

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_sgetvector_async(
    magma_int_t m, magma_int_t n,
    magmaFloat_const_ptr dA_src, size_t dA_offset, magma_int_t ldda,
    float*          hA_dst, size_t hA_offset, magma_int_t ldha,
    magma_queue_t queue, magma_event_t *event )
{
    magma_mic_memcpy2D_async(
        hA_dst + hA_offset, ldha*sizeof(float),
        dA_src + dA_offset, ldda*sizeof(float),
        m*sizeof(float), n,
        READFROM_DEV, queue, event );
    return MAGMA_SUCCESS;
}


// ========================================
// copying sub-matrices (contiguous columns)
extern "C" magma_int_t
magma_ssetmatrix(
    magma_int_t m, magma_int_t n,
    float const* hA_src, size_t hA_offset, magma_int_t ldha,
    magmaFloat_ptr    dA_dst, size_t dA_offset, magma_int_t ldda,
    magma_queue_t queue )
{
    magma_event_t event;
    magma_mic_synchronize_kernels();
    magma_mic_memcpy2D_async(
        dA_dst + dA_offset, ldda*sizeof(float),
        hA_src + hA_offset, ldha*sizeof(float),
        m*sizeof(float), n,
        WRITETO_DEV, queue, &event );
    magma_mic_fence_wait( event );
    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_sgetmatrix(
    magma_int_t m, magma_int_t n,
    magmaFloat_const_ptr dA_src, size_t dA_offset, magma_int_t ldda,
    float*          hA_dst, size_t hA_offset, magma_int_t ldha,
    magma_queue_t queue )
{
    magma_event_t event;
    magma_mic_synchronize_kernels();
    magma_mic_memcpy2D_async(
        hA_dst + hA_offset, ldha*sizeof(float),
        dA_src + dA_offset, ldda*sizeof(float),
        m*sizeof(float), n,
        READFROM_DEV, queue, &event );
    magma_mic_fence_wait( event );
    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_ssetmatrix_async(
    magma_int_t m, magma_int_t n,
    float const* hA_src, size_t hA_offset, magma_int_t ldha,
    magmaFloat_ptr    dA_dst, size_t dA_offset, magma_int_t ldda,
    magma_queue_t queue, magma_event_t *event )
{
    if ( m < 1 || n < 1)
        return MAGMA_SUCCESS;

    magma_mic_memcpy2D_async(
        dA_dst + dA_offset, ldda*sizeof(float),
        hA_src + hA_offset, ldha*sizeof(float),
        m*sizeof(float), n,
        WRITETO_DEV, queue, event );
    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_sgetmatrix_async(
    magma_int_t m, magma_int_t n,
    magmaFloat_const_ptr dA_src, size_t dA_offset, magma_int_t ldda,
    float*          hA_dst, size_t hA_offset, magma_int_t ldha,
    magma_queue_t queue, magma_event_t *event )
{
    if ( m < 1 || n < 1) 
        return MAGMA_SUCCESS;

    magma_mic_memcpy2D_async(
        hA_dst + hA_offset, ldha*sizeof(float),
        dA_src + dA_offset, ldda*sizeof(float),
        m*sizeof(float), n,
        READFROM_DEV, queue, event );
    return MAGMA_SUCCESS;
}

// --------------------
// TODO this doesn't match naming scheme; use magma_isetvector and magma_igetvector
extern "C" magma_int_t
magma_ssetintarray_async(
    magma_int_t lda, magma_int_t const* hA_src, size_t hA_offset,
    magma_int_t *dA_dst, size_t dA_offset, magma_queue_t queue, magma_event_t *event )
{
    magma_int_t ncol = 1;
    magma_mic_memcpy2D_async(
        (void *)(dA_dst + dA_offset), lda * sizeof(magma_int_t),
        (void *)(hA_src + hA_offset), lda * sizeof(magma_int_t),
        lda * sizeof(magma_int_t), ncol,
        WRITETO_DEV, queue, event);
    return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magma_sgetintarray_async(
    magma_int_t lda, magma_int_t const* hA_src, size_t hA_offset,
    magma_int_t *dA_dst, size_t dA_offset, magma_queue_t queue, magma_event_t *event )
{
    magma_mic_memcpy2D_async(
        (void *)(dA_dst + dA_offset), lda * sizeof(magma_int_t),
        (void *)(hA_src + hA_offset), lda * sizeof(magma_int_t),
        lda * sizeof(magma_int_t), 1,
        READFROM_DEV, queue, event);
    return MAGMA_SUCCESS;
}



extern "C" magma_int_t
magma_sprbt(
    magma_int_t n, 
    magmaFloat_ptr dA, magma_int_t ldda, 
    magmaFloat_const_ptr du, magmaFloat_const_ptr dv, 
    magma_queue_t handle)
{
	int err;
	magma_mic_sprbt_param param;

	param.n = n;
	param.d_A = dA;
	param.lda = ldda;
	param.d_u = du;
	param.d_v = dv;
	int control_msg = magma_mic_SPRBT;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magma_sprbt_mtv(
    magma_int_t n, 
    magmaFloat_const_ptr du, magmaFloat_ptr db, 
    magma_queue_t handle)
{
	int err;
	magma_mic_sprbt_mtv_param param;

	param.n = n;
	param.d_u = du;
	param.d_b = db;
	int control_msg = magma_mic_SPRBT_MTV;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

	
extern "C" magma_int_t
magma_sprbt_mv(
    magma_int_t n, 
    magmaFloat_const_ptr dv, magmaFloat_ptr db, 
    magma_queue_t handle)
{
	int err;
	magma_mic_sprbt_mv_param param;

	param.n = n;
	param.d_v = dv;
	param.d_b = db;
	int control_msg = magma_mic_SPRBT_MV;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}




// ========================================
// BLAS functions
// --------------------


extern "C" magma_int_t
magma_scsrgemv(
    magma_trans_t  trans,
    magma_int_t n,
    magmaFloat_const_ptr daval,
    magmaIndex_const_ptr darow,
    magmaIndex_const_ptr dacol,
    magmaFloat_const_ptr dx,
    magmaFloat_ptr dy,
    magma_queue_t handle )
{
    int err;
    magma_mic_scsrgemv_param param;

    param.transa = trans;
    param.m      = n;
    param.a      = daval;
    param.ia     = darow;
    param.ja     = dacol;   
    param.x      = dx + 0;
    param.y      = dy + 0;

    int control_msg = magma_mic_SCSRGEMV;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magmablas_scsrgemv(
    magma_trans_t  trans,
    magma_int_t n,
    magmaFloat_const_ptr daval,
    magmaIndex_const_ptr darow,
    magmaIndex_const_ptr dacol,
    magmaFloat_const_ptr dx,
    magmaFloat_ptr dy,
    magma_queue_t handle )
{
    int err;
    magma_mic_scsrgemv_param param;

    param.transa = trans;
    param.m      = n;
    param.a      = daval;
    param.ia     = darow;
    param.ja     = dacol;
    param.x      = dx + 0;
    param.y      = dy + 0;

    int control_msg = magmablas_mic_SCSRGEMV;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magma_scsrgespmm(
    magma_trans_t  trans,
    magma_int_t request,
    magma_int_t sort,
    magma_int_t m,
    magma_int_t n,
    magma_int_t k,
    float alpha,
    magmaFloat_const_ptr daval,
    magmaIndex_const_ptr darow,
    magmaIndex_const_ptr dacol,
    magmaFloat_const_ptr dbval,
    magmaIndex_const_ptr dbrow,
    magmaIndex_const_ptr dbcol,
    magmaFloat_ptr dcval,
    magmaIndex_ptr dcrow,
    magmaIndex_ptr dccol,
    magma_int_t nzmax,
    magma_int_t info,
    magma_queue_t handle )
{
    int err;
    magma_mic_scsrgespmm_param param;

    param.transa = trans;
    param.request= request;
    param.sort   = sort;
    param.m      = m;
    param.n      = n;
    param.k      = k;
    param.alpha  = alpha;
    param.a      = daval;
    param.ia     = darow;
    param.ja     = dacol;  
    param.b      = dbval;
    param.ib     = dbrow;
    param.jb     = dbcol;  
    param.c      = dcval;
    param.ic     = dcrow;
    param.jc     = dccol;  
    param.nzmax  = nzmax;
    param.info   = info;
    
    int control_msg = magma_mic_SCSRGESPMM;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}


extern "C" float
magma_sdot(
    magma_int_t n,
    magmaFloat_const_ptr dx,
    magma_int_t incx,
    magmaFloat_const_ptr dy,
    magma_int_t incy,
    magma_queue_t handle )
{
    int err;
    magma_mic_sdot_param param;

    magmaFloat_ptr hres, dres;
    float result;
    magma_smalloc_pinned(&hres, 1);
    magma_smalloc(&dres, 1);

    param.res    = dres; 
    param.n      = n;
    param.x      = dx + 0;
    param.incx   = incx;
    param.y      = dy + 0;
    param.incy    = incy;

    int control_msg = magma_mic_SDOT;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    magma_mic_synchronize_kernels();
    magma_sgetvector(1, dres, 0, hres, 0, handle);

    result = *hres;

    magma_free_pinned(hres);
    magma_free(dres);
    return result;
}


extern "C" float
magma_snrm2(
    magma_int_t n,
    magmaFloat_const_ptr dx,
    magma_int_t incx,
    magma_queue_t handle )
{
    int err;
    magma_mic_snrm2_param param;

    float *hres, *dres;
    float result;
    magma_smalloc_pinned(&hres, 1);
    magma_smalloc(&dres, 1);

    param.res    = dres;
    param.n      = n;
    param.x      = dx + 0;
    param.incx   = incx;

    int control_msg = magma_mic_SNRM2;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    magma_mic_synchronize_kernels();
    magma_sgetvector(1, dres, 0, hres, 0, handle);

    result = *hres;

    magma_free_pinned(hres);
    magma_free(dres);
    return result;
}


extern "C" magma_int_t
magma_saxpy(
    magma_int_t n,
    float alpha,
    magmaFloat_const_ptr dx,
    magma_int_t incx,
    magmaFloat_ptr dy,
    magma_int_t incy,
    magma_queue_t handle )
{
    int err;
    magma_mic_saxpy_param param;

    param.n      = n;
    param.a      = alpha;
    param.x      = dx + 0;
    param.incx   = incx;
    param.y      = dy + 0;
    param.incy    = incy;

    int control_msg = magma_mic_SAXPY;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}



extern "C" magma_int_t
magma_scopy(
    magma_int_t n,
    magmaFloat_const_ptr dx, magma_int_t incx,
    magmaFloat_ptr dy, magma_int_t incy,
    magma_queue_t handle )
{
    int err;
    magma_mic_scopy_param param;

    param.n      = n;
    param.x      = dx;
    param.incx   = incx;
    param.y      = dy;
    param.incy    = incy;

    int control_msg = magma_mic_SCOPY;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_scopyvector(
    magma_int_t n,
    magmaFloat_const_ptr dx_src, magma_int_t incx,
    magmaFloat_ptr dy_dst, magma_int_t incy,
    magma_queue_t handle )
{
    return magma_scopy(n, dx_src, incx, dy_dst, incy, handle); 
}



// --------------------
extern "C" magma_int_t
magma_sgemm(
    magma_trans_t transA, magma_trans_t transB,
    magma_int_t m, magma_int_t n, magma_int_t k,
    float alpha, magmaFloat_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloat_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    float beta,  magmaFloat_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t handle )
{
    int err;
    magma_mic_sgemm_param param;

    param.transa = transA;
    param.transb = transB;
    param.m      = m;
    param.n      = n;
    param.k      = k;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.b      = dB + dB_offset;
    param.ldb    = ldb;
    param.beta   = beta;
    param.c      = dC + dC_offset;
    param.ldc    = ldc;

    int control_msg = magma_mic_SGEMM;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_sgemv(
    magma_trans_t transA,
    magma_int_t m, magma_int_t n,
    float alpha, magmaFloat_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloat_const_ptr dx, size_t dx_offset, magma_int_t incx,
    float beta,  magmaFloat_ptr       dy, size_t dy_offset, magma_int_t incy,
    magma_queue_t handle )
{
    int err;
    magma_mic_sgemv_param param;

    param.transa = transA;
    param.m      = m;
    param.n      = n;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.x      = dx + dx_offset;
    param.incx   = incx;
    param.beta   = beta;
    param.y      = dy + dy_offset;
    param.incy    = incy;

    int control_msg = magma_mic_SGEMV;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_ssymm(
    magma_side_t side, magma_uplo_t uplo,
    magma_int_t m, magma_int_t n,
    float alpha, magmaFloat_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloat_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    float beta,  magmaFloat_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t handle )
{
    int err;
    magma_mic_ssymm_param param;

    param.side   = side;
    param.uplo   = uplo;
    param.m      = m;
    param.n      = n;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.b      = dB + dB_offset;
    param.ldb    = ldb;
    param.beta   = beta;
    param.c      = dC + dC_offset;
    param.ldc    = ldc;
    param.agboff = 0;

    int control_msg = magma_mic_SSYMM;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}
// --------------------
extern "C" magma_int_t
magma_ssymm_ours(
    magma_side_t side, magma_uplo_t uplo,
    magma_int_t m, magma_int_t n,
    float alpha, magmaFloat_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloat_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    float beta,  magmaFloat_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_int_t globaldAoffset,
    magma_queue_t handle )
{
    int err;
    magma_mic_ssymm_param param;

    param.side   = side;
    param.uplo   = uplo;
    param.m      = m;
    param.n      = n;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.b      = dB + dB_offset;
    param.ldb    = ldb;
    param.beta   = beta;
    param.c      = dC + dC_offset;
    param.ldc    = ldc;
    param.ldc    = ldc;
    param.agboff = globaldAoffset;

    int control_msg = magma_mic_SSYMM;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_sscal(
    magma_int_t n,
    float alpha,
    float *dx, magma_int_t incx,
    magma_queue_t handle )
{
    int err;
    magma_mic_sscal_param param = {n, alpha, dx, incx};

    int control_msg = magma_mic_SSCAL;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}


// --------------------
extern "C" magma_int_t
magma_ssymv2(
    magma_uplo_t uplo,
    magma_int_t n,
    float alpha, magmaFloat_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloat_const_ptr dx, size_t dx_offset, magma_int_t incx,
    float beta,  magmaFloat_ptr       dy, size_t dy_offset, magma_int_t incy,
    magma_queue_t handle )
{
    int err;
    magma_mic_ssymv2_param param;

    param.uplo   = uplo;
    param.n      = n;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.x      = dx + dx_offset;
    param.incx   = incx;
    param.beta   = beta;
    param.y      = dy + dy_offset;
    param.incy   = incy;

    int control_msg = magma_mic_SSYMV2;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_ssymv(
    magma_uplo_t uplo,
    magma_int_t n,
    float alpha, magmaFloat_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloat_const_ptr dx, size_t dx_offset, magma_int_t incx,
    float beta,  magmaFloat_ptr       dy, size_t dy_offset, magma_int_t incy,
    magma_queue_t handle )
{
    int err;
    magma_mic_ssymv_param param;

    param.uplo   = uplo;
    param.n      = n;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.x      = dx + dx_offset;
    param.incx   = incx;
    param.beta   = beta;
    param.y      = dy + dy_offset;
    param.incy    = incy;

    int control_msg = magma_mic_SSYMV;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

#ifdef PRECISION_d
// --------------------
extern "C" magma_int_t
magma_ssymv_for_trd(
    magma_uplo_t uplo,
    magma_int_t n,
    float alpha, magmaFloat_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloat_const_ptr dx, size_t dx_offset, magma_int_t incx,
    float beta,  magmaFloat_ptr       dy, size_t dy_offset, magma_int_t incy,
    magma_int_t symvshift, magmaFloat_const_ptr dpartialY, magma_queue_t handle )
{
    int err;
    magma_mic_ssymv_for_trd_param param;

    param.uplo   = uplo;
    param.n      = n;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.x      = dx + dx_offset;
    param.incx   = incx;
    param.beta   = beta;
    param.y      = dy + dy_offset;
    param.incy    = incy;
    param.symvshift = symvshift;
    param.partialY = dpartialY;

    int control_msg = magma_mic_SSYMVFORTRD;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

#endif

// --------------------
extern "C" magma_int_t
magma_ssyrk(
    magma_uplo_t uplo,
    magma_trans_t trans,
    magma_int_t n, magma_int_t k,
    float alpha, magmaFloat_const_ptr dA, size_t dA_offset, magma_int_t lda,
    float beta,  magmaFloat_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t handle )
{
    int err;
    magma_mic_ssyrk_param param;

    param.uplo  = uplo;
    param.trans = trans;
    param.n     = n;
    param.k     = k;
    param.alpha = alpha;
    param.a     = dA + dA_offset;
    param.lda   = lda;
    param.beta  = beta;
    param.c     = dC + dC_offset;
    param.ldc   = ldc;

    int control_msg = magma_mic_SSYRK;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}


// --------------------
extern "C" magma_int_t
magma_ssyrk_mmic(
    magma_uplo_t uplo, magma_trans_t trans,
    magma_int_t n, magma_int_t k,
    float alpha, magmaFloat_const_ptr dA, magma_int_t lda, magma_int_t dAoff,
    float beta,  magmaFloat_const_ptr dC, magma_int_t ldc, magma_int_t offset,
    magma_int_t nmic, magma_int_t nb, magma_int_t my_id )
{
    int err;
    magma_mic_ssyrk_mmic_param param;

    param.uplo   = uplo;
    param.trans  = trans;
    param.n      = n;
    param.k      = k;
    param.alpha  = alpha;
    param.a      = dA;
    param.lda    = lda;
    param.aoff   = dAoff;
    param.beta   = beta;
    param.c      = dC;
    param.ldc    = ldc;
    param.offset = offset;
    param.nb     = nb;
    param.nmic   = nmic;
    param.my_id  = my_id;

    int control_msg = magma_mic_SSYRK_MMIC;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}



// --------------------
extern "C" magma_int_t
magma_ssyr2k(
    magma_uplo_t uplo, magma_trans_t trans,
    magma_int_t n, magma_int_t k,
    float alpha, magmaFloat_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloat_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    float              beta, magmaFloat_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t queue )
{
    int err;
    magma_mic_ssyr2k_param param;

    param.uplo  = uplo;
    param.trans = trans;
    param.n     = n;
    param.k     = k;
    param.alpha = alpha;
    param.a     = dA + dA_offset;
    param.lda   = lda;
    param.b     = dB + dB_offset;
    param.ldb   = ldb;
    param.beta  = beta;
    param.c     = dC + dC_offset;
    param.ldc   = ldc;

    int control_msg = magma_mic_SSYR2K;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_ssyr2k_mmic(
    magma_uplo_t uplo, magma_trans_t trans, magma_int_t n, magma_int_t k,
    float alpha, magmaFloat_const_ptr dA, magma_int_t lda, magma_int_t dAoff,
                              magmaFloat_const_ptr dB, magma_int_t ldb, magma_int_t dBoff,
    float beta,              magmaFloat_const_ptr dC, magma_int_t ldc,  magma_int_t offset,
    magma_int_t nmic, magma_int_t nb, magma_int_t my_id)
{
    int err;
    magma_mic_ssyr2k_mmic_param param;

    param.uplo   = uplo;
    param.trans  = trans;
    param.n      = n;
    param.k      = k;
    param.alpha  = alpha;
    param.a      = dA;
    param.lda    = lda;
    param.aoff   = dAoff;
    param.b      = dB;
    param.ldb    = ldb;
    param.boff   = dBoff;
    param.beta   = beta;
    param.c      = dC;
    param.ldc    = ldc;
    param.offset = offset;
    param.nb     = nb;
    param.nmic   = nmic;
    param.my_id  = my_id;

    int control_msg = magma_mic_SSYR2K_MMIC;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}


// --------------------
extern "C" magma_int_t
magma_strsm(
    magma_side_t  side,
    magma_uplo_t  uplo,
    magma_trans_t trans,
    magma_diag_t  diag,
    magma_int_t m, magma_int_t n,
    float alpha, magmaFloat_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloat_ptr       dB, size_t dB_offset, magma_int_t ldb,
    magma_queue_t handle )
{
    int err;
    magma_mic_strsm_param param;

    param.side   = side;
    param.uplo   = uplo;
    param.transa = trans;
    param.diag   = diag;
    param.m      = m;
    param.n      = n;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.b      = dB + dB_offset;
    param.ldb    = ldb;

    int control_msg = magma_mic_STRSM;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_strsv(magma_uplo_t uplo, magma_trans_t trans, magma_diag_t diag,
    magma_int_t n,
    magmaFloat_const_ptr dA, size_t dA_offset, magma_int_t lda,
    magmaFloat_ptr dx, size_t dx_offset, magma_int_t incx,
    magma_queue_t queue )
{
    int err;
    magma_mic_strsv_param param;
    
    param.uplo   = uplo;
    param.trans  = trans;
    param.diag   = diag;
    param.n      = n;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.dx     = dx + dx_offset;
    param.incx   = incx;
    
    int control_msg = magma_mic_STRSV;
    
    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    
    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    
    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_strmm(
    magma_side_t side, magma_uplo_t uplo, magma_trans_t trans, magma_diag_t diag,
    magma_int_t m, magma_int_t n,
    float alpha, magmaFloat_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloat_ptr       dB, size_t dB_offset, magma_int_t ldb,
    magma_queue_t queue )
{
    int err;
    magma_mic_strmm_param param;

    param.side   = side;
    param.uplo   = uplo;
    param.transa = trans;
    param.diag   = diag;
    param.m      = m;
    param.n      = n;
    param.alpha  = alpha;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.b      = dB + dB_offset;
    param.ldb    = ldb;

    int control_msg = magma_mic_STRMM;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magma_strmv(magma_uplo_t uplo, magma_trans_t trans, magma_diag_t diag,
    magma_int_t n,
    magmaFloat_const_ptr dA, size_t dA_offset, magma_int_t lda,
    magmaFloat_ptr dx, size_t dx_offset, magma_int_t incx,
    magma_queue_t queue )
{
    int err;
    magma_mic_strmv_param param;
    
    param.uplo   = uplo;
    param.trans  = trans;
    param.diag   = diag;
    param.n      = n;
    param.a      = dA + dA_offset;
    param.lda    = lda;
    param.dx     = dx + dx_offset;
    param.incx   = incx;
    
    int control_msg = magma_mic_STRMV;
    
    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    
    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    
    return MAGMA_SUCCESS;
}




// ========================================
// Auxuliary routines

extern "C" magma_int_t
magma_sdebug(
               magma_int_t n,
               magmaFloat_const_ptr daval,
               magmaIndex_const_ptr darow,
               magmaIndex_const_ptr dacol,
               magmaFloat_ptr dx,
               magmaFloat_const_ptr db,
               magmaFloat_ptr dwork,
               magma_queue_t handle )
{
    int err;
    magma_mic_sdebug_param param;

    param.m      = n;
    param.a      = daval;
    param.ia     = darow;
    param.ja     = dacol;
    param.x      = dx + 0;
    param.b      = db + 0;
    param.dwork  = dwork;

    int control_msg = magma_mic_SDEBUG;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magma_stranspose(magmaFloat_ptr odata, magma_int_t offo, magma_int_t ldo,
    magmaFloat_ptr idata, magma_int_t offi, magma_int_t ldi,
    magma_int_t m, magma_int_t n,
    magma_queue_t queue)
{
    int err;
    magma_mic_stranspose_param param;

    param.odata = odata + offo;
    param.ldo   = ldo;
    param.idata = idata + offi;
    param.ldi   = ldi;
    param.m     = m;
    param.n     = n;

    int control_msg = magma_mic_STRANSPOSE;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magma_slaswp(
    magma_int_t n, magmaFloat_ptr d_A, magma_int_t lda,
    magma_int_t k1, magma_int_t k2, magma_int_t *d_ipiv, magma_int_t incx,
    magma_queue_t queue)
{
    magma_event_t event;
    int err;
    magma_mic_slaswp_param param;
    param.n      = n;
    param.d_A    = d_A;
    param.lda    = lda;
    param.k1     = k1;
    param.k2     = k2;
    param.d_ipiv = d_ipiv;
    param.incx   = incx;

    int control_msg = magma_mic_SLASWP;
    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magma_sswap(
    magma_int_t n, float *dA1T, magma_int_t lda1,
    float *dA2T, magma_int_t lda2,
    magma_queue_t queue)
{
    magma_event_t event;
    int err;
    magma_mic_sswap_param param;
    param.n      = n;
    param.dA1T    = dA1T;
    param.lda1    = lda1;
    param.dA2T    = dA2T;
    param.lda2    = lda2;

    int control_msg = magma_mic_SSWAP;
    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magma_slasyf_mic(
    magma_uplo_t uplo, magma_int_t n, magma_int_t nb, magma_int_t *kb,
    float *hA, magma_int_t lda,
    magmaFloat_ptr dA, size_t dA_offset, magma_int_t ldda,
    magma_int_t *d_ipiv,
    magmaFloat_ptr dW, size_t dW_offset, magma_int_t lddw,
    magma_int_t *info, magma_queue_t queue )
{
    int err;
    magma_mic_slasyf_param param = {uplo, n, nb, kb, hA, lda,
				    dA+dA_offset, ldda, d_ipiv, dW+dW_offset, lddw};

    int control_msg = magma_mic_SLASYF;
    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
       err = errno;
       printf("scif_send failed with err %d\n", errno);
       fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
       err = errno;
       printf("scif_send failed with err %d\n", errno);
       fflush(stdout);
    }
    return MAGMA_SUCCESS;
}

// This version sends a larfb request to the MIC. The difference with the version in
// 'src' is that here there is only one request and control is passed immediately back
// to the CPU.
extern "C" magma_int_t
magma_slarfb(
    magma_side_t side, magma_trans_t trans, magma_direct_t direct, magma_storev_t storev,
    magma_int_t m, magma_int_t n, magma_int_t k,
    magmaFloat_ptr dV, size_t dV_offset, magma_int_t ldv,
    magmaFloat_ptr dT, size_t dT_offset, magma_int_t ldt,
    magmaFloat_ptr dC, size_t dC_offset, magma_int_t ldc,
    magmaFloat_ptr dwork, size_t dwork_offset, magma_int_t ldwork,
    magma_queue_t queue)
{
    int err;
    magma_mic_slarfb_param param;

    param.side   = side;
    param.trans  = trans;
    param.direct = direct;
    param.storev = storev;
    param.m      = m;
    param.n      = n;
    param.k      = k;
    param.v      = dV + dV_offset;
    param.ldv    = ldv;
    param.t      = dT + dT_offset;
    param.ldt    = ldt;
    param.c      = dC + dC_offset;
    param.ldc    = ldc;
    param.work   = dwork + dwork_offset;
    param.ldwork = ldwork;

    int control_msg = magma_mic_SLARFB;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magma_slarfb_gemm(
    magma_side_t side, magma_trans_t trans, magma_direct_t direct, magma_storev_t storev,
    magma_int_t m, magma_int_t n, magma_int_t k,
    magmaFloat_ptr dV, size_t dV_offset, magma_int_t ldv,
    magmaFloat_ptr dT, size_t dT_offset, magma_int_t ldt,
    magmaFloat_ptr dC, size_t dC_offset, magma_int_t ldc,
    magmaFloat_ptr dwork, size_t dwork_offset, magma_int_t ldwork,
    magmaFloat_ptr dworkvt, size_t dworkvt_offset, magma_int_t ldworkvt,
    magma_queue_t queue)
{
    int err;
    magma_mic_slarfb_gemm_param larfb_gemm_param;

    larfb_gemm_param.side   = side;
    larfb_gemm_param.trans  = trans;
    larfb_gemm_param.direct = direct;
    larfb_gemm_param.storev = storev;
    larfb_gemm_param.m      = m;
    larfb_gemm_param.n      = n;
    larfb_gemm_param.k      = k;
    larfb_gemm_param.v      = dV + dV_offset;
    larfb_gemm_param.ldv    = ldv;
    larfb_gemm_param.t      = dT + dT_offset;
    larfb_gemm_param.ldt    = ldt;
    larfb_gemm_param.c      = dC + dC_offset;
    larfb_gemm_param.ldc    = ldc;
    larfb_gemm_param.work   = dwork + dwork_offset;
    larfb_gemm_param.ldwork = ldwork;
    larfb_gemm_param.workvt   = dworkvt + dworkvt_offset;
    larfb_gemm_param.ldworkvt = ldworkvt;

    int control_msg = magma_mic_SLARFB_GEMM;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &larfb_gemm_param, sizeof(larfb_gemm_param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magma_slaset(magma_uplo_t uplo,
             magma_int_t m, magma_int_t n,
             float alpha, float beta,
             magmaFloat_ptr dA, size_t dA_offset, magma_int_t lda,
             magma_queue_t queue)
{
     int err;
     magma_mic_slaset_param param;
     int control_msg = magma_mic_SLASET;

     param.uplo   = uplo;
     param.m      = m;
     param.n      = n;
     param.alpha  = alpha;
     param.beta   = beta;
     param.dA     = dA + dA_offset;
     param.lda    = lda;

     if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
         err = errno;
         printf("scif_send failed with err %d\n", errno);
         fflush(stdout);
     }

     if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
     }

     return MAGMA_SUCCESS;
}


// ========================================
// BLAS functions - combined blas for MIC
extern "C" magma_int_t
magma_strsmgemm(
    magma_side_t side, magma_uplo_t uplo,
    magma_trans_t trans, magma_diag_t diag,
    magma_int_t tm, magma_int_t tn,
    float talpha, magmaFloat_const_ptr tdA, size_t tdA_offset, magma_int_t tlda,
                               magmaFloat_ptr       tdB, size_t tdB_offset, magma_int_t tldb,
    
    magma_trans_t transA, magma_trans_t transB,
    magma_int_t m, magma_int_t n, magma_int_t k,
    float alpha, magmaFloat_const_ptr dA, size_t dA_offset, magma_int_t lda,
                              magmaFloat_const_ptr dB, size_t dB_offset, magma_int_t ldb,
    float beta,  magmaFloat_ptr       dC, size_t dC_offset, magma_int_t ldc,
    magma_queue_t handle )
{
    int err;
    magma_mic_strsmgemm_param param;

    //trsm parameter
    param.trsm_param.side   = side;
    param.trsm_param.uplo   = uplo;
    param.trsm_param.transa = trans;
    param.trsm_param.diag   = diag;
    param.trsm_param.m      = tm;
    param.trsm_param.n      = tn;
    param.trsm_param.alpha  = talpha;
    param.trsm_param.a      = tdA + tdA_offset;
    param.trsm_param.lda    = tlda;
    param.trsm_param.b      = tdB + tdB_offset;
    param.trsm_param.ldb    = tldb;

    //gemm parameter
    param.gemm_param.transa = transA;
    param.gemm_param.transb = transB;
    param.gemm_param.m      = m;
    param.gemm_param.n      = n;
    param.gemm_param.k      = k;
    param.gemm_param.alpha  = alpha;
    param.gemm_param.a      = dA + dA_offset;
    param.gemm_param.lda    = lda;
    param.gemm_param.b      = dB + dB_offset;
    param.gemm_param.ldb    = ldb;
    param.gemm_param.beta   = beta;
    param.gemm_param.c      = dC + dC_offset;
    param.gemm_param.ldc    = ldc;

    int control_msg = magma_mic_STRSM_GEMM;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

extern "C" float
magmablas_slange(
    magma_norm_t norm, magma_int_t m, magma_int_t n,
    magmaFloat_const_ptr dA, size_t dA_offset, magma_int_t ldda,
    magmaFloat_ptr dwork,
    magma_queue_t handle)
{
    int err;
    magma_mic_slange_param param;
    
    float *ret, *dret;
    float ret2;
    
    magma_smalloc_pinned(&ret, 1);
    magma_smalloc(&dret, 1);
  
    magma_event_t event; 

    param.norm   = norm ;   
    param.m      = m    ;    
    param.n      = n    ;    
    param.dA     = dA + dA_offset;   
    param.ldda   = ldda ;      
    param.dwork  = dwork;  
    param.ret    = dret;
    
    int control_msg = magma_mic_SLANGE;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    magma_mic_synchronize_kernels();
#define PRECISION_s
#if defined PRECISION_z | defined PRECISION_d
    magma_sgetvector(1, dret, 0, ret, 0, handle);
#else
    magma_sgetvector(1, dret, 0, ret, 0, handle);
#endif
    ret2 = *ret;
    magma_free_pinned(ret);
    magma_free(dret);

    return ret2;
}

extern "C" float
magmablas_slansy(
    magma_norm_t norm, magma_uplo_t uplo, magma_int_t n,
    magmaFloat_const_ptr dA, size_t dA_offset, magma_int_t ldda,
    magmaFloat_ptr dwork,
    magma_queue_t handle)
{
    int err;
    magma_mic_slansy_param param;
    
    float *ret, *dret;
    float ret2;
    
    magma_smalloc_pinned(&ret, 1);
    magma_smalloc(&dret, 1);
  
    magma_event_t event; 

    param.norm   = norm ;
    param.uplo   = uplo ;
    param.n      = n    ;    
    param.dA     = dA + dA_offset;   
    param.ldda   = ldda ;      
    param.dwork  = dwork;  
    param.ret    = dret;
    
    int control_msg = magma_mic_SLANSY;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    magma_mic_synchronize_kernels();
#define PRECISION_s
#if defined PRECISION_z | defined PRECISION_d
    magma_sgetvector(1, dret, 0, ret, 0, handle);
#else
    magma_sgetvector(1, dret, 0, ret, 0, handle);
#endif
    ret2 = *ret;
    
    magma_free_pinned(ret);
    magma_free(dret);
    return ret2;
}

// --------------------

extern "C" magma_int_t
magmablas_slacpy(
    magma_uplo_t uplo, magma_int_t m, magma_int_t n,
    magmaFloat_const_ptr dA, magma_int_t ldda,
    magmaFloat_ptr dB, magma_int_t lddb,
    magma_queue_t handle)
{
    int err;
    magma_mic_slacpy_param param;
    
    
    param.uplo   = uplo ;   
    param.m      = m    ;    
    param.n      = n    ;    
    param.dA     = dA   ;   
    param.ldda   = ldda ;      
    param.dB     = dB   ;   
    param.lddb   = lddb ;      
    
    int control_msg = magma_mic_SLACPY;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------
extern "C" magma_int_t
magmablas_slascl_diag(
    magma_uplo_t uplo, magma_int_t m, magma_int_t n,
    magmaFloat_ptr dA, magma_int_t ldda,
    magmaFloat_ptr dB, magma_int_t lddb,
    magma_queue_t handle)
{
    int err;
    magma_mic_slascl_diag_param param;

    param.uplo   = uplo ;
    param.m      = m    ;
    param.n      = n    ;
    param.dA     = dA   ;
    param.ldda   = ldda ;
    param.dB     = dB   ;
    param.lddb   = lddb ;

    int control_msg = magma_mic_SLASCL_DIAG;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

// --------------------

extern "C" magma_int_t
magma_isamax(
    magma_int_t n,
    magmaFloat_const_ptr dx, magma_int_t incx,
    magma_queue_t handle)
{
    int err;
    magma_mic_isamax_param param;
   
    magma_int_t *ret;
    magma_int_t *dret;
    magma_int_t ret2;

    magma_mic_synchronize_kernels();

    //magma_imalloc_pinned(&ret, 1);
    //magma_mic_synchronize_kernels();
    magma_malloc_pinned((void **) &ret, sizeof(magma_int_t));

    //magma_imalloc(&dret, 1);
    //magma_mic_synchronize_kernels();
    magma_malloc((void **)&dret, sizeof(magma_int_t));
        
    magma_event_t event; 

    param.n      = n    ;    
    param.dx     = dx   ;   
    param.incx   = incx ;      
    param.ret    = dret ;
    
    int control_msg = magma_mic_ISAMAX;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    magma_mic_synchronize_kernels();

    //magma_sgetintarray_async(1, dret, 1, ret, 1, handle, &event);  
    //magma_event_sync(event);
    //magma_mic_synchronize_kernels();
    magma_getvector(1, sizeof(magma_int_t), dret, 1, ret, 1, NULL);

    ret2 = *ret;

    magma_free_pinned(ret);
    magma_free(dret);
    return ret2;
}

extern "C" magma_int_t
magmablas_saxpycp2(
    magma_int_t m,
    magmaFloat_ptr r, magmaFloat_ptr x,magmaFloat_ptr b,
    magma_queue_t handle)
{
    int err;
    magma_mic_saxpycp2_param param;
    
    
    param.m      = m   ;    
    param.r      = r   ;   
    param.x      = x   ;   
    param.b      = b   ;   
    
    int control_msg = magma_mic_SAXPYCP2;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}



// SPARSE MAGMA
// ========================================

// CG on MIC-only
extern "C" magma_int_t
magma_scg_mic(
    magma_int_t n, magmaFloat_const_ptr daval, magmaIndex_const_ptr darow,
    magmaIndex_const_ptr dacol, magmaFloat_const_ptr db, magmaFloat_ptr dx,
    magma_s_solver_par *solver_par, magmaFloat_ptr dwork)
{
    int err;

    magma_s_solver_par *dpars;
    magma_malloc((void**)&dpars, sizeof(magma_s_solver_par));

    // verbose is not supported yet
    if (solver_par->verbose != 0){
        printf("Verbose is not supported in scg_mic. Switching it to 0.\n");
        solver_par->verbose = 0;
    }    

    magma_setvector(1, sizeof(magma_s_solver_par), solver_par, 1, dpars, 1, NULL);
    magma_mic_scg_param param = {n, daval, darow, dacol, dx, db, dpars, dwork};

    int control_msg = magma_mic_SCG;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    magma_getvector(1, sizeof(magma_s_solver_par), dpars, 1, solver_par, 1, NULL);

    magma_free(dpars);

    return MAGMA_SUCCESS;
}

// BICGSTAB on MIC-only 
extern "C" magma_int_t
magma_sbicgstab_mic(
    magma_int_t n, magmaFloat_const_ptr daval, magmaIndex_const_ptr darow,
    magmaIndex_const_ptr dacol, magmaFloat_const_ptr db, magmaFloat_ptr dx,
    magma_s_solver_par *solver_par, magmaFloat_ptr dwork)
{
    int err;

    magma_s_solver_par *dpars;
    magma_malloc((void**)&dpars, sizeof(magma_s_solver_par));

    // verbose is not supported yet
    if (solver_par->verbose != 0){
        printf("Verbose is not supported in scg_mic. Switching it to 0.\n");
        solver_par->verbose = 0;
    }

    magma_setvector(1, sizeof(magma_s_solver_par), solver_par, 1, dpars, 1, NULL);

    magma_mic_sbicgstab_param param = {n, daval, darow, dacol, dx, db, dpars, dwork};

    int control_msg = magma_mic_SBICGSTAB;
    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    magma_getvector(1, sizeof(magma_s_solver_par), dpars, 1, solver_par, 1, NULL);
    magma_free(dpars);

    return MAGMA_SUCCESS;
}

// GMRES on MIC-only
extern "C" magma_int_t
magma_sgmres_mic(
    magma_int_t n, magmaFloat_const_ptr daval, magmaIndex_const_ptr darow,
    magmaIndex_const_ptr dacol, magmaFloat_const_ptr db, magmaFloat_ptr dx,
    magma_s_solver_par *solver_par, magmaFloat_ptr dwork)
{
    int err;

    magma_s_solver_par *dpars;
    magma_malloc((void**)&dpars, sizeof(magma_s_solver_par));

    // verbose is not supported yet
    if (solver_par->verbose != 0){
        printf("Verbose is not supported in scg_mic. Switching it to 0.\n");
        solver_par->verbose = 0;
    }

    magma_setvector(1, sizeof(magma_s_solver_par), solver_par, 1, dpars, 1, NULL);

    magma_mic_sgmres_param param = {n, daval, darow, dacol, dx, db, dpars, dwork};

    int control_msg = magma_mic_SGMRES;
    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    magma_getvector(1, sizeof(magma_s_solver_par), dpars, 1, solver_par, 1, NULL);
    magma_free(dpars);

    return MAGMA_SUCCESS;
}

// Jacobi precond application
extern "C" magma_int_t
magma_sjacobidiagscale(
    magma_int_t n,
    magma_s_vector d,
    magma_s_vector b,
    magma_s_vector *c,
    magma_queue_t queue )
{
    int err;
    magma_mic_sjacobidiagscale_param param;

    param.m      = n;
    param.n      = b.num_rows*b.num_cols/n;
    param.b      = b.dval;
    param.d      = d.dval;
    param.c      = c->dval;
    
    int control_msg = magma_mic_SJACOBIDIAGSCALE;

    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &param, sizeof(param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    return MAGMA_SUCCESS;
}

#endif // HAVE_MIC
