/*
 *   -- micMAGMA (version 1.4.0) --
 *      Univ. of Tennessee, Knoxville
 *      Univ. of California, Berkeley
 *      Univ. of Colorado, Denver
 *      @date July 2015
 *
 * @precisions mixed zc -> ds
 */

// defines stub functions, if no GPU implementation is available.
#include <stdio.h>
#include <stdlib.h>

#include "magmasparse.h"
#include "magma.h"

#ifdef HAVE_MIC

#include "magma_mic_bindings.h"


extern "C" magma_int_t
magma_zlag2c( magma_int_t m, magma_int_t n,
              magmaDoubleComplex_ptr dA, size_t dA_offset, magma_int_t lda,
              magmaFloatComplex_ptr dAs, size_t dAs_offset, magma_int_t ldas,
              magma_queue_t queue)
{
  magma_event_t event;
  int err;
  magma_mic_zlag2c_param zlag2c_param;

  zlag2c_param.m                  =       m;
  zlag2c_param.n                  =       n;
  zlag2c_param.dA                 =       dA + dA_offset;
  zlag2c_param.lda                =       lda;
  zlag2c_param.dAs                =       dAs + dAs_offset;
  zlag2c_param.ldas               =       ldas;

  int control_msg         =       magma_mic_ZLAG2C;
  if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
    err = errno;
    printf("scif_send failed with err %d\n", errno);
    fflush(stdout);
  }

  if ((err = scif_send(gEpd, &zlag2c_param, sizeof(zlag2c_param), 1)) <= 0) {
    err = errno;
    printf("scif_send failed with err %d\n", errno);
    fflush(stdout);
  }
  return MAGMA_SUCCESS;
}

extern "C" magma_int_t
magma_zlat2c( magma_uplo_t uplo, magma_int_t n,
              magmaDoubleComplex_ptr dA, size_t dA_offset, magma_int_t lda,
              magmaFloatComplex_ptr dAs, size_t dAs_offset, magma_int_t ldas,
              magma_queue_t queue)
{
    magma_event_t event;
    int err;
    magma_mic_zlat2c_param zlat2c_param;

    zlat2c_param.uplo               =       uplo;
    zlat2c_param.n                  =       n;
    zlat2c_param.dA                 =       dA + dA_offset;
    zlat2c_param.lda                =       lda;
    zlat2c_param.dAs                =       dAs + dAs_offset;
    zlat2c_param.ldas               =       ldas;

    int control_msg         =       magma_mic_ZLAT2C;
    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &zlat2c_param, sizeof(zlat2c_param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    return MAGMA_SUCCESS;
}


extern "C" magma_int_t
magma_clag2z( magma_int_t m, magma_int_t n,
              magmaFloatComplex_ptr dAs, size_t dAs_offset, magma_int_t ldas,
              magmaDoubleComplex_ptr dA, size_t dA_offset, magma_int_t lda,
              magma_queue_t queue)
{
  magma_event_t event;
  int err;
  magma_mic_clag2z_param clag2z_param;

  clag2z_param.m                  =       m;
  clag2z_param.n                  =       n;
  clag2z_param.dAs                =       dAs + dAs_offset;
  clag2z_param.ldas               =       ldas;
  clag2z_param.dA                 =       dA + dA_offset;
  clag2z_param.lda                =       lda;

  int control_msg         =       magma_mic_CLAG2Z;
  if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
    err = errno;
    printf("scif_send failed with err %d\n", errno);
    fflush(stdout);
  }

  if ((err = scif_send(gEpd, &clag2z_param, sizeof(clag2z_param), 1)) <= 0) {
    err = errno;
    printf("scif_send failed with err %d\n", errno);
    fflush(stdout);
  }
  return MAGMA_SUCCESS;
}


extern "C" magma_int_t
magma_zcaxpycp( magma_int_t m,
                magmaFloatComplex_ptr r, size_t r_offset,
                magmaDoubleComplex_ptr x, size_t x_offset,
                magmaDoubleComplex_const_ptr b, size_t b_offset,
                magmaDoubleComplex_ptr w, size_t w_offset,
                magma_queue_t queue )
{
    magma_event_t event;
    int err;
    magma_mic_zcaxpycp_param zcaxpycp_param;

    zcaxpycp_param.m                  =       m;
    zcaxpycp_param.r                  =       r + r_offset;
    zcaxpycp_param.x                  =       x + x_offset;
    zcaxpycp_param.b                  =       b + b_offset;
    zcaxpycp_param.w                  =       w + w_offset;

    int control_msg         =       magma_mic_ZCAXPYCP;
    if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }

    if ((err = scif_send(gEpd, &zcaxpycp_param, sizeof(zcaxpycp_param), 1)) <= 0) {
        err = errno;
        printf("scif_send failed with err %d\n", errno);
        fflush(stdout);
    }
    return MAGMA_SUCCESS;
}


extern "C" magma_int_t
magma_zcgesv(magma_int_t N, magma_int_t NRHS,
             magmaDoubleComplex_ptr dA, size_t dA_offset, magma_int_t ldda,
             magma_int_t *dIPIV,
             magmaDoubleComplex_ptr dB, size_t dB_offset, magma_int_t lddb,
             magmaDoubleComplex_ptr dX, size_t dX_offset, magma_int_t lddx,
             magmaDoubleComplex_ptr dworkd, magmaFloatComplex_ptr dworks,
             magma_queue_t queue )
{
  magma_event_t event;
  int err;
  magma_mic_zcgesv_param zcgesv_param;

  zcgesv_param.n      = N;
  zcgesv_param.nrhs   = NRHS;
  zcgesv_param.d_A    = dA + dA_offset;
  zcgesv_param.lda    = ldda;
  zcgesv_param.d_ipiv = dIPIV;
  zcgesv_param.d_B    = dB + dB_offset;
  zcgesv_param.ldb    = lddb;
  zcgesv_param.d_X    = dX + dX_offset;
  zcgesv_param.ldx    = lddx;
  zcgesv_param.d_work = dworkd;
  zcgesv_param.d_swork= dworks;

  int control_msg     = magma_mic_ZCGESV;
  if ((err = scif_send(gEpd, &control_msg, sizeof(control_msg), 1)) <= 0) {
    err = errno;
    printf("scif_send failed with err %d\n", errno);
    fflush(stdout);
  }

  if ((err = scif_send(gEpd, &zcgesv_param, sizeof(zcgesv_param), 1)) <= 0) {
    err = errno;
    printf("scif_send failed with err %d\n", errno);
    fflush(stdout);
  }

  magma_mic_synchronize_kernels();

  return MAGMA_SUCCESS;
}


#endif // HAVE_MIC
