/*
    -- micMAGMA (version 1.4.0) --
       Univ. of Tennessee, Knoxville
       Univ. of California, Berkeley
       Univ. of Colorado, Denver
       @date July 2015

       @generated from zjacobisetup.cpp normal z -> d, Sun Jul 12 17:46:28 2015
       @author Hartwig Anzt

*/

#include "magmasparse.h"
#include "magma.h"
#include "magma_d_server.h"

//#include "common_magma.h"
//#include "magmasparse.h"


void 
djacobidiagscale_kernel(  
                    int grid, 
                    int threads,
                    int num_rows,
                    int num_vecs, 
                    magmaDouble_ptr b, 
                    magmaDouble_ptr d, 
                    magmaDouble_ptr c){

    //#pragma omp parallel for    
    for(int blockIdx=0; blockIdx < grid; blockIdx++) {
        for(int threadIdx=0; threadIdx < threads; threadIdx++) {
            c[ blockIdx+threadIdx*num_rows ] = b[ blockIdx+threadIdx*num_rows ] * d[ blockIdx ];
        }
    }
    
}





/**
    Purpose
    -------

    Prepares the Jacobi Iteration according to
       x^(k+1) = D^(-1) * b - D^(-1) * (L+U) * x^k
       x^(k+1) =      c     -       M        * x^k.

    Returns the vector c. It calls a GPU kernel

    Arguments
    ---------

    @param[in]
    num_rows    magma_int_t
                number of rows
                
    @param[in]
    b           magma_d_vector
                RHS b

    @param[in]
    d           magma_d_vector
                vector with diagonal entries

    @param[out]
    c           magma_d_vector*
                c = D^(-1) * b
    @param[in]
    queue       magma_queue_t
                Queue to execute in.

    @ingroup magmasparse_d
    ********************************************************************/

void
call_djacobidiagscale( magma_mic_djacobidiagscale_param param ){

    magmaDouble_ptr b = param.b; 
    magmaDouble_ptr d = param.d; 
    magmaDouble_ptr c = param.c;
    
    int num_rows = param.m; 
    int num_vecs = param.n;
    
    int grid = num_rows;
    int threads = num_vecs ;

    djacobidiagscale_kernel( grid, threads, num_rows, num_vecs, d, b, c );

}



