#ifndef MAGMA_Z_SERVER_H
#define MAGMA_Z_SERVER_H

/*
 *   -- micMAGMA (version 1.4.0) --
 *      Univ. of Tennessee, Knoxville
 *      Univ. of California, Berkeley
 *      Univ. of Colorado, Denver
 *      April 2012
 *
 *   @author Azzam Haidar
 *   @author Khairul Kabir
 *   @author Stan Tomov
 *
 *   @precisions normal z -> s d c
 */

#include <mkl.h>

#include <mkl_trans.h>
#include <scif.h>
#include <fcntl.h>
#include <offload.h>

#if 0
#ifdef __MIC__
#define _M512_TYPED
#include <lmmintrin.h>
#endif
#endif
//#include <immintrin.h>

#include "magma.h"
#include "magma_z_bindings.h"
#include "magmasparse.h"

// Flops formula
#define MICFMULS_GEMM(m, n, k) ((m) * (n) * (k))
#define MICFADDS_GEMM(m, n, k) ((m) * (n) * (k))
#define MICFMULS_SYMM(side, m, n) (((side) == MagmaLeft) ? MICFMULS_GEMM(m, m, n) : MICFMULS_GEMM(m, n, n))
#define MICFADDS_SYMM(side, m, n) (((side) == MagmaLeft) ? MICFADDS_GEMM(m, m, n) : MICFADDS_GEMM(m, n, n))
#define MICFMULS_GEMV(m, n)   ((m) * (n))
#define MICFADDS_GEMV(m, n)   ((m) * (n))
#define MICFMULS_TRMM_L(m, n) (0.5 * (n) * (m) * ((m)+1))
#define MICFADDS_TRMM_L(m, n) (0.5 * (n) * (m) * ((m)-1))
#define MICFMULS_TRMM_R(m, n) (0.5 * (m) * (n) * ((n)+1))
#define MICFADDS_TRMM_R(m, n) (0.5 * (m) * (n) * ((n)-1))
#define MICFMULS_TRSM_L(m, n) (0.5 * (n) * (m) * ((m)+1))
#define MICFADDS_TRSM_L(m, n) (0.5 * (n) * (m) * ((m)-1))
#define MICFMULS_TRSM_R(m, n) (0.5 * (m) * (n) * ((n)+1))
#define MICFADDS_TRSM_R(m, n) (0.5 * (m) * (n) * ((n)-1))
#define MICFMULS_SYRK(k, n) (0.5 * (k) * (n) * ((n)+1))
#define MICFADDS_SYRK(k, n) (0.5 * (k) * (n) * ((n)+1))
#define MICFMULS_SYR2K(k, n) ((k) * (n) * (n)        )
#define MICFADDS_SYR2K(k, n) ((k) * (n) * (n) + (n)  )

#define MICFLOPS_GEMM(m, n, k) (MICFMULS_GEMM(m, n, k) + MICFADDS_GEMM(m, n, k))
#define MICFLOPS_SYMM(side, m, n) (MICFMULS_SYMM(side, m, n) + MICFADDS_SYMM(side, m, n))
#define MICFLOPS_GEMV(m, n)    (MICFMULS_GEMV(m, n) + MICFADDS_GEMV(m, n))
#define MICFLOPS_TRSM_L(m, n)  (MICFMULS_TRSM_L(m, n) + MICFADDS_TRSM_L(m, n))
#define MICFLOPS_TRSM_R(m, n)  (MICFMULS_TRSM_R(m, n) + MICFADDS_TRSM_R(m, n))

#define MICFLOPS_SYRK(k, n)    (MICFMULS_SYRK(k, n) + MICFADDS_SYRK(k, n))
#define MICFLOPS_SYR2K(k, n)   (MICFMULS_SYR2K(k, n) + MICFADDS_SYR2K(k, n))

void call_zcopy(magma_mic_zcopy_param param);

void call_zgemm(magma_mic_zgemm_param param);

void call_zhemm(magma_mic_zhemm_param param);

void call_zgemv(magma_mic_zgemv_param param);

void call_zhemv(magma_mic_zhemv_param param);

void call_zhemv_for_trd(magma_mic_zhemv_for_trd_param param);

void call_zprbt(magma_mic_zprbt_param param);

void call_zprbt_mtv(magma_mic_zprbt_mtv_param param);

void call_zprbt_mv(magma_mic_zprbt_mv_param param);

void call_zsymv2(magma_mic_zsymv2_param param);

void call_ztrsm(magma_mic_ztrsm_param param);

void call_ztrsv(magma_mic_ztrsv_param param);

void call_ztrmv(magma_mic_ztrmv_param param);

void call_ztrmm(magma_mic_ztrmm_param param);

void call_zherk(magma_mic_zherk_param param);

void call_zherk_mmic(magma_mic_zherk_mmic_param param);

void call_zher2k(magma_mic_zher2k_param param);

void call_zher2k_mmic(magma_mic_zher2k_mmic_param param);

void call_zlahef(magma_mic_zlahef_param param);

void call_zlarft(magma_mic_zlarft_param param);

void call_zlarfb(magma_mic_zlarfb_param param);

void call_zlarfb_gemm(magma_mic_zlarfb_gemm_param param);

void call_zlaset(magma_mic_zlaset_param param);

void call_ztranspose(magma_mic_ztranspose_param param);

void call_ztranspose_mkl(magma_mic_ztranspose_param param);

void call_zlaswp(magma_mic_zlaswp_param param);

void call_zswap(magma_mic_zswap_param param);

void call_ztrsmgemm(magma_mic_ztrsmgemm_param param);

void call_zlange(magma_mic_zlange_param param);

void call_zlacpy(magma_mic_zlacpy_param param);

void call_zlascl_diag(magma_mic_zlascl_diag_param param);

void call_izamax(magma_mic_izamax_param param);

void call_zaxpycp2(magma_mic_zaxpycp2_param param);

void call_zjacobidiagscale( magma_mic_zjacobidiagscale_param param );

#define SREG_WIDTH 16
#define DREG_WIDTH 8

#define _MM512_LOAD_PPS _mm512_load_ps
#define _MM512_LOAD_PPD _mm512_load_pd

#define _MM512_STORE_PPS _mm512_store_ps
#define _MM512_STORE_PPD _mm512_store_pd

#define _MM512_I32LOEXTSCATTER_PPS _mm512_i32extscatter_ps
#define _MM512_I32LOEXTSCATTER_PPD _mm512_i32loextscatter_pd

#define __M512S __m512
#define __M512D __m512d

#ifdef __cplusplus
extern "C" {
#endif

// MAGMA MIC Sparse Native kernels
magma_int_t magma_zcg_mic(
    magma_int_t n, magmaDoubleComplex_const_ptr daval, magmaIndex_const_ptr darow,
    magmaIndex_const_ptr dacol, magmaDoubleComplex_const_ptr db, magmaDoubleComplex_ptr dx,
    magma_z_solver_par *solver_par, magmaDoubleComplex_ptr dwork);

magma_int_t magma_zbicgstab_mic(
    magma_int_t n, magmaDoubleComplex_const_ptr daval, magmaIndex_const_ptr darow,
    magmaIndex_const_ptr dacol,magmaDoubleComplex_const_ptr db, magmaDoubleComplex_ptr dx,
    magma_z_solver_par *solver_par, magmaDoubleComplex_ptr dwork);

magma_int_t magma_zgmres_mic(
    magma_int_t n, magmaDoubleComplex_const_ptr daval, magmaIndex_const_ptr darow,
    magmaIndex_const_ptr dacol,magmaDoubleComplex_const_ptr db, magmaDoubleComplex_ptr dx,
    magma_z_solver_par *solver_par, magmaDoubleComplex_ptr dwork);

#ifdef __cplusplus
}
#endif

#endif // MAGMA_Z_SERVER_H
