/*
 *   -- micMAGMA (version 1.4.0) --
 *      Univ. of Tennessee, Knoxville
 *      Univ. of California, Berkeley
 *      Univ. of Colorado, Denver
 *      April 2012
 *
 *   @precisions mixed zc -> ds
 *
 *   This is included in magma_mic_server.cpp,
 *   inside a switch { } block in the do_work() function.
 */

case magma_mic_ZCAXPYCP:
{
    magma_mic_zcaxpycp_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZCAXPYCP\n");
        return;
    }

    mic_time = dsecnd();
    #pragma omp parallel for schedule(static,48)
    for (int i=0; i<param.m; i++){
        param.x[i] += param.r[i];
        param.w[i]  = param.b[i];
    }

    mic_time = dsecnd() - mic_time;

    break;
}

case magma_mic_ZLAG2C:
{
    magma_mic_zlag2c_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZLAG2C\n");
        return;
    }
    
    mic_time = dsecnd();
    call_zlag2c(param);
    mic_time = dsecnd() - mic_time;
    
    break;
}

case magma_mic_ZLAT2C:
 {
     magma_int_t info;
     magma_mic_zlat2c_param param;
     if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
         printf("scif_recv failed for ZLAT2C\n");
         return;
     }

     mic_time = dsecnd();
     zlat2c(lapack_uplo_const(param.uplo), &param.n, 
            (MKL_Complex16 *)param.dA, &param.lda, 
            (MKL_Complex8 *)param.dAs, &param.ldas, &info);
     mic_time = dsecnd() - mic_time;

     break;
 }

case magma_mic_CLAG2Z:
{
    magma_mic_clag2z_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for CLAG2Z\n");
        return;
    }
    
    mic_time = dsecnd();
    call_clag2z(param);
    mic_time = dsecnd() - mic_time;
    
    break;
}


case magma_mic_ZCGESV:
{
    magma_mic_zcgesv_param param;
    if (mic_recv(&param, sizeof(param)) == MIC_FAILURE) {
        printf("scif_recv failed for ZCGESV\n");
        return;
    }
    
    mic_time = dsecnd();
    call_zcgesv(param);
    mic_time = dsecnd() - mic_time;
    
    break;
}
