/*
    -- MAGMA (version 1.4.0) --
       Univ. of Tennessee, Knoxville
       Univ. of California, Berkeley
       Univ. of Colorado, Denver
       @date July 2015

       @generated from magma_zmshift.cpp normal z -> d, Sun Jul 12 17:46:30 2015
       @author Hartwig Anzt
*/

#include <fstream>
#include <stdlib.h>
#include <string>
#include <sstream>
#include <iostream>
#include <ostream>
#include <assert.h>
#include <stdio.h>
#include "magmasparse_d.h"
#include "magma.h"
#include "mmio.h"



using namespace std;

/**
    Purpose
    -------

    Shifts all rowpointers and colindices by s to be usable by Intel MKL.


    Arguments
    ---------

    @param[in][out]
    A           magma_d_sparse_matrix*
                sparse matrix A
                
    @param[in]
    s           magma_int_t
                all indices are shifted by s
   
    @param[in]
    queue       magma_queue_t
                Queue to execute in.

    @ingroup magmasparse_daux
    ********************************************************************/

extern "C" magma_int_t
magma_dmshift(
    magma_d_sparse_matrix *A,
    magma_int_t s,
    magma_queue_t queue )
{
    // set queue for old dense routines
    //magma_queue_t orig_queue;
    //magmablasGetKernelStream( &orig_queue );

    magma_int_t stat_cpu = 0, stat_dev = 0;

    // make sure matrix is on CPU
    if( A->memory_location == Magma_CPU ){
            // check format
            // CSR type
            if( A->storage_type == Magma_CSR ){
                    for( magma_int_t i=0; i<A->num_rows+1; i++ ){
                            A->row[ i ] = A->row[ i ] + s;
                    }
                    for( magma_int_t i=0; i<A->nnz; i++ ){
                            A->col[ i ] = A->col[ i ] + s;
                    }
            }
            else{
                printf("error: format not supported\n");
                return MAGMA_ERR_NOT_IMPLEMENTED;
            }
    }
    else{
        magma_d_sparse_matrix ACPU;
        magma_location_t memory_location = A->memory_location;
        magma_d_mtransfer( *A, &ACPU, A->memory_location, Magma_CPU, queue );
        magma_d_mfree( A, queue );
        magma_dmshift( &ACPU, s, queue );
        magma_d_mtransfer( ACPU, A, Magma_CPU, memory_location, queue );
        magma_d_mfree( &ACPU, queue );
    }
    return MAGMA_SUCCESS;
}

