/*
    -- MAGMA (version 1.4.0) --
       Univ. of Tennessee, Knoxville
       Univ. of California, Berkeley
       Univ. of Colorado, Denver
       @date July 2015

       @generated from magma_z_solver_wrapper.cpp normal z -> c, Sun Jul 12 17:46:31 2015
       @author Hartwig Anzt

*/
#include "common_magma.h"
#include "magmasparse.h"




/**
    Purpose
    -------

    ALlows the user to choose a solver.

    Arguments
    ---------

    @param[in]
    A           magma_c_sparse_matrix
                sparse matrix A    

    @param[in]
    b           magma_c_vector
                input vector b     

    @param[in]
    x           magma_c_vector*
                output vector x        

    @param[in]
    zopts     magma_copts
              options for solver and preconditioner
    @param[in]
    queue       magma_queue_t
                Queue to execute in.

    @ingroup magmasparse_caux
    ********************************************************************/

extern "C" magma_int_t
magma_c_solver(
    magma_c_sparse_matrix A, magma_c_vector b, 
    magma_c_vector *x, magma_copts *zopts,
    magma_queue_t queue )
{
    // preconditioner
        if ( zopts->solver_par.solver != Magma_ITERREF ) {
            magma_c_precondsetup( A, b, &zopts->precond_par, queue );
        }
        switch( zopts->solver_par.solver ) {
            case  Magma_CG:
                magma_ccg( A, b, x, &zopts->solver_par, queue );break;
            case  Magma_CG_mic:
                {
                    magmaFloatComplex_ptr dwork;
                    magma_cmalloc(&dwork, 3*A.num_rows);
                    magma_ccg_mic(A.num_rows,
                                  A.dval, A.drow, A.dcol,
                                  b.dval, x->dval, &zopts->solver_par, dwork);
                    magma_free(dwork);
                    break;
                }
            case  Magma_CGMERGE:
                printf("error: not supported\n");break;//  magma_ccg_merge( A, b, x, &zopts->solver_par, queue );break;
            case  Magma_PCG:
                    magma_cpcg( A, b, x, &zopts->solver_par, &zopts->precond_par, queue );break;
            case  Magma_BICGSTAB:
                      magma_cbicgstab( A, b, x, &zopts->solver_par, queue );break;
            case  Magma_BICGSTAB_mic:
                {
                    magmaFloatComplex_ptr dwork;
                    magma_cmalloc(&dwork, 6*A.num_rows);
                    magma_cbicgstab_mic(A.num_rows,
                                        A.dval, A.drow, A.dcol,
                                        b.dval, x->dval, &zopts->solver_par, dwork);
                    magma_free(dwork);
                    break;
                }
            case  Magma_BICGSTABMERGE: 
                    printf("error: not supported\n");break;//  magma_cbicgstab_merge( A, b, x, &zopts->solver_par, queue );break;
            case  Magma_PBICGSTAB: 
                  break;//  magma_cpbicgstab( A, b, x, &zopts->solver_par, &zopts->precond_par, queue );break;
            case  Magma_GMRES: 
                      magma_cfgmres( A, b, x, &zopts->solver_par, &zopts->precond_par, queue );break;
            case  Magma_GMRES_mic:
                {
                    magmaFloatComplex_ptr dwork;
                    magma_int_t r = zopts->solver_par.restart;
                    magma_cmalloc(&dwork, (r+3)*A.num_rows + (r+1)*(5+2*r));
                    magma_cgmres_mic(A.num_rows,
                                     A.dval, A.drow, A.dcol,
                                     b.dval, x->dval, &zopts->solver_par, dwork);
                    magma_free(dwork);
                    break;
                }
            case  Magma_PGMRES: 
                      magma_cfgmres( A, b, x, &zopts->solver_par, &zopts->precond_par, queue );break;
            case  Magma_LOBPCG: 
                  break;//  magma_clobpcg( A, &zopts->solver_par, queue );break;
            case  Magma_ITERREF:
                    printf("error: not supported\n");break;//  magma_citerref( A, b, x, &zopts->solver_par, &zopts->precond_par, queue );break;
            case  Magma_JACOBI: 
                    magma_cjacobi( A, b, x, &zopts->solver_par, queue );break;
            case  Magma_BAITER: 
                    printf("error: not supported\n");break;//  magma_cbaiter( A, b, x, &zopts->solver_par, queue );break;
            
        }
    return MAGMA_SUCCESS;
}


