/*
    -- MAGMA (version 1.4.0) --
       Univ. of Tennessee, Knoxville
       Univ. of California, Berkeley
       Univ. of Colorado, Denver
       @date July 2015

       @generated from magma_z_solver_wrapper.cpp normal z -> s, Sun Jul 12 17:46:31 2015
       @author Hartwig Anzt

*/
#include "common_magma.h"
#include "magmasparse.h"




/**
    Purpose
    -------

    ALlows the user to choose a solver.

    Arguments
    ---------

    @param[in]
    A           magma_s_sparse_matrix
                sparse matrix A    

    @param[in]
    b           magma_s_vector
                input vector b     

    @param[in]
    x           magma_s_vector*
                output vector x        

    @param[in]
    zopts     magma_sopts
              options for solver and preconditioner
    @param[in]
    queue       magma_queue_t
                Queue to execute in.

    @ingroup magmasparse_saux
    ********************************************************************/

extern "C" magma_int_t
magma_s_solver(
    magma_s_sparse_matrix A, magma_s_vector b, 
    magma_s_vector *x, magma_sopts *zopts,
    magma_queue_t queue )
{
    // preconditioner
        if ( zopts->solver_par.solver != Magma_ITERREF ) {
            magma_s_precondsetup( A, b, &zopts->precond_par, queue );
        }
        switch( zopts->solver_par.solver ) {
            case  Magma_CG:
                magma_scg( A, b, x, &zopts->solver_par, queue );break;
            case  Magma_CG_mic:
                {
                    magmaFloat_ptr dwork;
                    magma_smalloc(&dwork, 3*A.num_rows);
                    magma_scg_mic(A.num_rows,
                                  A.dval, A.drow, A.dcol,
                                  b.dval, x->dval, &zopts->solver_par, dwork);
                    magma_free(dwork);
                    break;
                }
            case  Magma_CGMERGE:
                printf("error: not supported\n");break;//  magma_scg_merge( A, b, x, &zopts->solver_par, queue );break;
            case  Magma_PCG:
                    magma_spcg( A, b, x, &zopts->solver_par, &zopts->precond_par, queue );break;
            case  Magma_BICGSTAB:
                      magma_sbicgstab( A, b, x, &zopts->solver_par, queue );break;
            case  Magma_BICGSTAB_mic:
                {
                    magmaFloat_ptr dwork;
                    magma_smalloc(&dwork, 6*A.num_rows);
                    magma_sbicgstab_mic(A.num_rows,
                                        A.dval, A.drow, A.dcol,
                                        b.dval, x->dval, &zopts->solver_par, dwork);
                    magma_free(dwork);
                    break;
                }
            case  Magma_BICGSTABMERGE: 
                    printf("error: not supported\n");break;//  magma_sbicgstab_merge( A, b, x, &zopts->solver_par, queue );break;
            case  Magma_PBICGSTAB: 
                  break;//  magma_spbicgstab( A, b, x, &zopts->solver_par, &zopts->precond_par, queue );break;
            case  Magma_GMRES: 
                      magma_sfgmres( A, b, x, &zopts->solver_par, &zopts->precond_par, queue );break;
            case  Magma_GMRES_mic:
                {
                    magmaFloat_ptr dwork;
                    magma_int_t r = zopts->solver_par.restart;
                    magma_smalloc(&dwork, (r+3)*A.num_rows + (r+1)*(5+2*r));
                    magma_sgmres_mic(A.num_rows,
                                     A.dval, A.drow, A.dcol,
                                     b.dval, x->dval, &zopts->solver_par, dwork);
                    magma_free(dwork);
                    break;
                }
            case  Magma_PGMRES: 
                      magma_sfgmres( A, b, x, &zopts->solver_par, &zopts->precond_par, queue );break;
            case  Magma_LOBPCG: 
                  break;//  magma_slobpcg( A, &zopts->solver_par, queue );break;
            case  Magma_ITERREF:
                    printf("error: not supported\n");break;//  magma_siterref( A, b, x, &zopts->solver_par, &zopts->precond_par, queue );break;
            case  Magma_JACOBI: 
                    magma_sjacobi( A, b, x, &zopts->solver_par, queue );break;
            case  Magma_BAITER: 
                    printf("error: not supported\n");break;//  magma_sbaiter( A, b, x, &zopts->solver_par, queue );break;
            
        }
    return MAGMA_SUCCESS;
}


