/*
    -- MAGMA (version 1.4.0) --
       Univ. of Tennessee, Knoxville
       Univ. of California, Berkeley
       Univ. of Colorado, Denver
       @date July 2015

       @generated from testing_zblas.cpp normal z -> c, Sun Jul 12 17:46:31 2015
       @author Hartwig Anzt
*/

// includes, system
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

// includes, project
#include "flops.h"
#include "magma.h"
#include "magmasparse.h"
#include "magma_lapack.h"
#include "testings.h"



/* ////////////////////////////////////////////////////////////////////////////
   -- testing any solver 
*/
int main(  int argc, char** argv )
{
    /* Initialize */
    magma_queue_t  queue;
    magma_device_t device;
    magma_int_t num = 0;
    magma_int_t err;

    magma_init();
    err = magma_get_devices( &device, 1, &num );
    if ( err != MAGMA_SUCCESS or num < 1 ) {
        fprintf( stderr, "magma_get_devices failed: %d\n", (int) err );
        exit(-1);
    }
    err = magma_queue_create( device, &queue );
    if ( err != MAGMA_SUCCESS ) {
        fprintf( stderr, "magma_queue_create failed: %d\n", (int) err );
        exit(-1);
    }

    magma_int_t j, n=1000000, FLOPS;
    
    magmaFloatComplex one = MAGMA_C_MAKE( 1.0, 0.0 );
    magmaFloatComplex two = MAGMA_C_MAKE( 2.0, 0.0 );

    magma_c_vector a, ad, bd, cd;
    magma_c_vinit( &a, Magma_CPU, n, one, queue );
    magma_c_vinit( &bd, Magma_DEV, n, two, queue );
    magma_c_vinit( &cd, Magma_DEV, n, one, queue );
    
    magma_c_vtransfer( a, &ad, Magma_CPU, Magma_DEV, queue ); 

    real_Double_t start, end, res;
    
    FLOPS = 2*n;
    start = magma_sync_wtime( queue );
    for (j=0; j<100; j++)
        res = magma_scnrm2(n, ad.dval, 1, queue );  
    end = magma_sync_wtime( queue );
    printf( " > MAGMA nrm2: %.2e seconds %.2e GFLOP/s\n",
                                    (end-start)/100, FLOPS*100/1e9/(end-start) );   
    printf("res: %f\n", res);

    FLOPS = n;
    start = magma_sync_wtime( queue );
    for (j=0; j<100; j++)
        magma_cscal( n, two, ad.dval, 1 , queue );    
    end = magma_sync_wtime( queue );
    printf( " > MAGMA scal: %.2e seconds %.2e GFLOP/s\n",
                                    (end-start)/100, FLOPS*100/1e9/(end-start) );   
    FLOPS = 2*n;
    start = magma_sync_wtime( queue );
    for (j=0; j<100; j++)
        magma_caxpy( n, one, ad.dval, 1, bd.dval, 1 , queue ); 
    end = magma_sync_wtime( queue );
    printf( " > MAGMA axpy: %.2e seconds %.2e GFLOP/s\n",
                                    (end-start)/100, FLOPS*100/1e9/(end-start) );   
    FLOPS = n;
    start = magma_sync_wtime( queue );
    for (j=0; j<100; j++)
        magma_ccopy( n, bd.dval, 1, ad.dval, 1 , queue ); 
    end = magma_sync_wtime( queue );
    printf( " > MAGMA copy: %.2e seconds %.2e GFLOP/s\n",
                                    (end-start)/100, FLOPS*100/1e9/(end-start) );
    FLOPS = 2*n;
    start = magma_sync_wtime( queue );
    for (j=0; j<100; j++) 
        res = MAGMA_C_REAL( magma_cdotc(n, ad.dval, 1, bd.dval, 1, queue ) );
    end = magma_sync_wtime( queue );
    printf( " > MAGMA dotc: %.2e seconds %.2e GFLOP/s\n",
                                    (end-start)/100, FLOPS*100/1e9/(end-start) );   

    printf("res: %f\n", res);


    magma_c_vfree( &a, queue);
    magma_c_vfree(&ad, queue);
    magma_c_vfree(&bd, queue);
    magma_c_vfree(&cd, queue);

    /* Shutdown */
    magma_queue_destroy( queue );
    magma_finalize();
    
    return 0;
}
