/*
    -- MAGMA (version 1.4.0) --
       Univ. of Tennessee, Knoxville
       Univ. of California, Berkeley
       Univ. of Colorado, Denver
       @date July 2015

       @generated from testing_zsolver.cpp normal z -> c, Sun Jul 12 17:46:31 2015
       @author Hartwig Anzt
       @author Stan Tomov
*/

// includes, system
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

// includes, project
#include "flops.h"
#include "magma.h"
#include "magmasparse.h"
#include "magma_lapack.h"
#include "testings.h"



/* ////////////////////////////////////////////////////////////////////////////
   -- testing any solver 
*/
int main(  int argc, char** argv )
{
    /* Initialize */
    magma_queue_t  queue;
    magma_device_t device;
    magma_int_t num = 0;
    magma_int_t err;

    magma_init();
    err = magma_get_devices( &device, 1, &num );
    if ( err != MAGMA_SUCCESS or num < 1 ) {
        fprintf( stderr, "magma_get_devices failed: %d\n", (int) err );
        exit(-1);
    }
    err = magma_queue_create( device, &queue );
    if ( err != MAGMA_SUCCESS ) {
        fprintf( stderr, "magma_queue_create failed: %d\n", (int) err );
        exit(-1);
    }

    magma_copts *zopts;
    magma_malloc_pinned((void**)&zopts, sizeof(magma_copts));
    
    int i=1;
    magma_cparse_opts( argc, argv, zopts, &i, queue );


    magmaFloatComplex one = MAGMA_C_MAKE(1.0, 0.0);
    magmaFloatComplex zero = MAGMA_C_MAKE(0.0, 0.0);
    magma_c_sparse_matrix A, B_d;
    magma_c_vector x, b;

    if ( zopts->solver_par.solver != Magma_PCG &&
         zopts->solver_par.solver != Magma_PGMRES &&
         zopts->solver_par.solver != Magma_PBICGSTAB &&
         zopts->solver_par.solver != Magma_ITERREF ) {
        zopts->precond_par.solver = Magma_NONE;
    }

    magma_csolverinfo_init( &(zopts->solver_par), &(zopts->precond_par), queue );

    while(  i < argc ) {

        if ( strcmp("LAPLACE2D", argv[i]) == 0 && i+1 < argc ) {   // Laplace test
            i++;
            magma_int_t laplace_size = atoi( argv[i] );
            magma_cm_5stencil(  laplace_size, &A, queue );
        } else {                        // file-matrix test
            magma_c_csr_mtx( &A,  argv[i], queue );
        }

        printf( "\n# matrix info: %d-by-%d with %d nonzeros\n\n",
                            (int) A.num_rows,(int) A.num_cols,(int) A.nnz );


        // for the eigensolver case
        zopts->solver_par.ev_length = A.num_rows;
        magma_ceigensolverinfo_init( &(zopts->solver_par), queue );

        magma_c_mtransfer( A, &B_d, Magma_CPU, Magma_DEV, queue );

        // vectors and initial guess
        magma_c_vinit( &b, Magma_DEV, A.num_cols, one, queue );
        magma_c_vinit( &x, Magma_DEV, A.num_cols, one, queue );
        magma_c_spmv( one, B_d, x, zero, b, queue );                 //  b = A x
        magma_c_vfree(&x, queue );
        magma_c_vinit( &x, Magma_DEV, A.num_cols, zero, queue );

        magma_c_solver( B_d, b, &x, zopts, queue );         

        magma_csolverinfo( &(zopts->solver_par), &(zopts->precond_par), queue );

        magma_c_mfree(&B_d, queue );
        magma_c_mfree(&A, queue ); 
        magma_c_vfree(&x, queue );
        magma_c_vfree(&b, queue );

        i++;
    }

    magma_csolverinfo_free( &(zopts->solver_par), &(zopts->precond_par), queue );
    
    /* Shutdown */
    magma_free_pinned(zopts);
    magma_queue_destroy( queue );
    magma_finalize();
    
    return 0;
}
