/*
    -- MAGMA (version 1.4.0) --
       Univ. of Tennessee, Knoxville
       Univ. of California, Berkeley
       Univ. of Colorado, Denver
       @date July 2015

       @generated from testing_zspmm_check.cpp normal z -> c, Sun Jul 12 17:46:31 2015
       @author Hartwig Anzt
*/

// includes, system
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

// includes, project
#include "flops.h"
#include "magma.h"
#include "magmasparse.h"
#include "magma_lapack.h"
#include "testings.h"



/* ////////////////////////////////////////////////////////////////////////////
   -- testing csr matrix matrix multiply
*/
int main(  int argc, char** argv )
{
    /* Initialize */
    magma_queue_t  queue;
    magma_device_t device;
    magma_int_t num = 0;
    magma_int_t err;

    magma_init();    
    err = magma_get_devices( &device, 1, &num );
    if ( err != MAGMA_SUCCESS or num < 1 ) {
        fprintf( stderr, "magma_get_devices failed: %d\n", (int) err );
        exit(-1);
    }
    err = magma_queue_create( device, &queue );
    if ( err != MAGMA_SUCCESS ) {
        fprintf( stderr, "magma_queue_create failed: %d\n", (int) err );
        exit(-1);
    }

    real_Double_t res;
    magma_c_sparse_matrix A, B, B2, C, A_d, B_d, C_d;

    magmaFloatComplex one = MAGMA_C_MAKE(1.0, 0.0);
    magmaFloatComplex zero = MAGMA_C_MAKE(0.0, 0.0);
    magmaFloatComplex mone = MAGMA_C_MAKE(-1.0, 0.0);

    magma_int_t i=1;

    if ( strcmp("LAPLACE2D", argv[i]) == 0 && i+1 < argc ) {   // Laplace test
        i++;
        magma_int_t laplace_size = atoi( argv[i] );
        magma_cm_5stencil(  laplace_size, &A, queue );
    } else {                        // file-matrix test
        magma_c_csr_mtx( &A,  argv[i], queue );
    }
    printf( "# matrix info: %d-by-%d with %d nonzeros\n",
                        (int) A.num_rows,(int) A.num_cols,(int) A.nnz );
    i++;

    if ( strcmp("LAPLACE2D", argv[i]) == 0 && i+1 < argc ) {   // Laplace test
        i++;
        magma_int_t laplace_size = atoi( argv[i] );
        magma_cm_5stencil(  laplace_size, &B, queue );
    } else {                        // file-matrix test
        magma_c_csr_mtx( &B,  argv[i], queue );
    }
    printf( "# matrix info: %d-by-%d with %d nonzeros\n",
                        (int) B.num_rows,(int) B.num_cols,(int) B.nnz );


    magma_c_mtransfer( A, &A_d, Magma_CPU, Magma_DEV, queue );
    magma_c_mtransfer( B, &B_d, Magma_CPU, Magma_DEV, queue );

    magma_c_spmm( one, A_d, B_d, &C_d, queue );

    magma_c_mvisu( A_d, queue );
    magma_c_mvisu( B_d, queue );
    magma_c_mvisu( C_d, queue );

    magma_c_mfree(&A_d, queue );
    magma_c_mfree(&B_d, queue );
    magma_c_mfree(&C_d, queue );
   
    magma_c_mfree(&A, queue ); 
    magma_c_mfree(&B, queue ); 

    /* Shutdown */
    magma_queue_destroy( queue );
    magma_finalize();

    return 0;
}
