/*
 *  -- clMAGMA (version 1.4.0) --
 *     Univ. of Tennessee, Knoxville
 *     Univ. of California, Berkeley
 *     Univ. of Colorado, Denver
 *     April 2012
 *
 *
 **/
// includes, system
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

// includes, project
//#include "../include/magmasparse.h"
//#include "../include/magmasparse_d.h"
//#include "../include/magmasparse_s.h"
//#include "../include/magmasparse_ds.h"
//#include "../../include/magma.h"
//#include "../include/mmio.h"
//#include "magma_lapack.h"
//#include "testings.h"


// includes, project
#include "../../include/magma.h"
#include "../include/magmasparse.h"
#include "../include/magmasparse_s.h"
#include "../include/magmasparse_d.h"
#include "../../include/magma_lapack.h"
#include "testings.h"

//#define PRECISION_d

int main( int argc, char** argv)
{
   printf("Hallo OpenCL\n");

  	/* Initialize */
	magma_queue_t  queue;
	magma_device_t device;
	int num = 0;
	magma_err_t err;
	magma_init();
	err = magma_get_devices( &device, 1, &num );
	if ( err != MAGMA_SUCCESS or num < 1 ) {
		fprintf( stderr, "magma_get_devices failed: %d\n", err );
		exit(-1);
	}
	err = magma_queue_create( device, &queue );
	if ( err != MAGMA_SUCCESS ) {
		fprintf( stderr, "magma_queue_create failed: %d\n", err );
		exit(-1);
	}

    magma_d_sparse_matrix A, B, C, D;
    magma_s_sparse_matrix As, Bs, Cs, Ds;
    magma_d_sparse_matrix_dev A_d, B_d, C_d, D_d;
    magma_s_sparse_matrix_dev As_d, Bs_d, Cs_d, Ds_d;
    A.storage_type = Magma_CSR;
    A.memory_location = Magma_CPU;
    A.num_rows = 20;
    A.num_cols = 20;
    A.nnz = 20;
    magma_malloc_pinned((void**) &A.val, A.num_rows*A.num_cols*sizeof(double) );
    magma_malloc_pinned((void**) &A.col, A.num_rows*A.num_cols*sizeof(magma_int_t) );
    magma_malloc_pinned((void**) &A.row, (A.num_rows+1)*sizeof(magma_int_t) );
    

    // initialize some example matrix
    for(int i=0; i<A.num_rows; i++){
        A.val[i] = 1.;
        A.col[i] = i;
        A.row[i] = i;
    }
    A.row[A.num_rows] = A.nnz;
    A.val[0] = 0.546897429867928469486948679486794867467;      
    printf("A: %d\n", A.memory_location );
    printf("double:%1.30f\n", A.val[0]);
    // bring it to GPU, convert it (on host) to dense 
    magma_dsetsparsematrix(  A, &A_d, Magma_CPU, Magma_DEV, queue );
    magma_d_mconvert_dev( A_d, &B_d, A.storage_type, Magma_DENSE, queue );
    magma_dgetsparsematrix(  B_d, &B, Magma_DEV, Magma_CPU, queue );


    // the factorization
    magma_int_t info, *ipiv;
    magma_imalloc_pinned(&ipiv, B_d.num_rows);
   magma_dgetrf_mic( B_d.num_rows, B_d.num_cols, B_d.val, 
                      0, B_d.num_rows, ipiv, &info, queue );



    magma_sparse_matrix_dlag2s( B_d, &Bs_d, queue );
    


    // move it back to CPU to print values
    magma_sgetsparsematrix(  Bs_d, &Bs, Magma_DEV, Magma_CPU,  queue);

    printf(" float:%1.30f\n", Bs.val[0]);


    // now for the vector
    // initialize random vector
    magma_d_vector rhs, x;
    magma_d_vector_dev rhs_d, x_d;
    magma_s_vector rhss;
    magma_s_vector_dev rhss_d;
    rhs.memory_location = Magma_CPU;
    rhs.num_rows = A.num_rows;
    magma_dmalloc_pinned(&(rhs.val), A.num_rows);
    for(int i=0; i<A.num_rows; i++){
        rhs.val[i] = 1.0;
    }

    // move to GPU
    magma_dsetsparsevector(  rhs, &rhs_d, Magma_CPU, Magma_DEV, queue );


    //convert to single precision
    magma_vector_dlag2s( rhs_d, &rhss_d, queue );


    magma_sgetrs_mic(MagmaNoTrans,  Bs_d.num_rows, 1, Bs_d.val, 0,  Bs_d.num_rows,
                     ipiv, rhss_d.val, 0, Bs_d.num_rows, &info, queue);


    //convert to double precision
    magma_vector_slag2d( rhss_d, &x_d, queue );

    // move back to CPU to print results
    magma_dgetsparsevector(  x_d, &x, Magma_DEV, Magma_CPU, queue );
    printf(" x[0] = %f\n", x.val[0]);



    printf("ende.\n");



}
