/*
 *  -- MAGMA (version 1.4.0) --
 *     Univ. of Tennessee, Knoxville
 *     Univ. of California, Berkeley
 *     Univ. of Colorado, Denver
 *     @date July 2015
 *
 * @generated from testing_zpotrf_mmic.cpp normal z -> s, Sun Jul 12 17:46:29 2015
 *
 **/
/* includes, system */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

/* includes, project */
#include "flops.h"
#include "magma.h"
#include "magma_lapack.h"
#include "magma_types.h"
#include "testings.h"

/* ////////////////////////////////////////////////////////////////////////////
   -- Testing spotrf_mmic
*/
int main( int argc, char** argv)
{
    /* Initialize */
    magma_queue_t  queue;
    magma_device_t device;
    magma_int_t num = 0;
    magma_int_t err;

    magma_init();
    err = magma_get_devices( &device, 1, &num );
    if ( err != MAGMA_SUCCESS or num < 1 ) {
        fprintf( stderr, "magma_get_devices failed: %d\n", (int) err );
        exit(-1);
    }
    err = magma_queue_create( device, &queue );
    if ( err != MAGMA_SUCCESS ) {
        fprintf( stderr, "magma_queue_create failed: %d\n", (int) err );
        exit(-1);
    }

    real_Double_t   gflops, mic_perf, mic_time, cpu_perf, cpu_time;
    float      work[1], error;
    float *h_A, *h_R;
    float *d_lA[4] = {NULL, NULL, NULL, NULL};
    magma_int_t N, n2, lda, ldda, info;
    magma_int_t i, j, k, num_mics0 = 1, num_mics;
    float c_neg_one = MAGMA_S_NEG_ONE;
    magma_int_t ione     = 1;
    magma_int_t ISEED[4] = {0,0,0,1};
    magma_int_t nb, nk, n_local, ldn_local;

    magma_opts opts;
    parse_opts( argc, argv, &opts );
    num_mics0 = opts.nmic;

    printf("  N     CPU GFlop/s (sec)   MAGMA GFlop/s (sec)   ||R_magma - R_lapack||_F / ||R_lapack||_F\n");
    printf("=============================================================================================\n");
    for(i = 0; i < opts.ntest; ++i ) {
        for( int iter = 0; iter < opts.niter; ++iter ) {
            N   = opts.nsize[i];
            lda = N;
            n2  = lda*N;
            ldda = ((N+31)/32)*32;
            gflops = FLOPS_SPOTRF( N ) / 1e9;

            TESTING_MALLOC_HOST(    h_A, float, n2     );
            TESTING_MALLOC_HOST( h_R, float, n2     );

            //TESTING_MALLOC_DEV(  d_A, float, ldda*N );
            nb = magma_get_spotrf_nb(N);
            if ( num_mics0 > N / nb ) {
                num_mics = N / nb;
                if ( N % nb != 0 ) num_mics++;
                printf( " * too many mics for the matrix size, using %d mics\n", (int) num_mics );
            } else {
                num_mics = num_mics0;
            }

            for(j = 0; j < num_mics; j++) {
                n_local = ((N / nb) / num_mics) * nb;
                if (j < (N / nb) % num_mics)
                    n_local += nb;
                else if (j == (N / nb) % num_mics)
                    n_local += N % nb;

                ldn_local = (n_local + 31);

                magma_setdevice(j);
                TESTING_MALLOC_DEV( d_lA[j], float, ldda * ldn_local );
            }

            /* Initialize the matrix */
            if (opts.check) {
                lapackf77_slarnv( &ione, ISEED, &n2, h_A );
                magma_smake_hpd( N, h_A, lda );
                lapackf77_slacpy( MagmaUpperLowerStr, &N, &N, h_A, &lda, h_R, &lda );
            } else {
                lapackf77_slarnv( &ione, ISEED, &n2, h_A );
                magma_smake_hpd( N, h_A, lda );
            }

            /* ====================================================================
               Performs operation using MAGMA
               =================================================================== */
            /* distribute matrix to mics */
            //if ( opts.uplo == MagmaUpper) {
                for(j = 0; j < N; j += nb) {
                    k = (j / nb) % num_mics;
                    magma_setdevice(k);
                    nk = min(nb, N - j);
                    magma_ssetmatrix( N, nk,
                            h_A + j * lda,  0,                     lda,
                            d_lA[k] + j / (nb * num_mics) * nb * ldda, 0, ldda, queue);
                }
            /*} else {
                for(j = 0; j < N; j += nb) {
                    k = (j / nb) % num_mics;
                    n_local = ((N / nb) / num_mics) * nb;
                    if (k < (N / nb) % num_mics)
                        n_local += nb;
                    else if (k == (N / nb) % num_mics)
                        n_local += N % nb;
                    ldn_local = ((n_local + 31) / 32) * 32;
                    ldn_local = (ldn_local % 256 == 0) ? ldn_local + 32 : ldn_local;

                    magma_setdevice(k);
                    nk = min(nb, N - j);
                    magma_ssetmatrix( nk, N,
                            h_A + j, 0,                     lda,
                            d_lA[k] + j / (nb * num_mics) * nb, 0, ldn_local, queue );
                }
            }*/

            mic_time = magma_wtime();
            magma_spotrf_mmic(num_mics, opts.uplo, N, d_lA, ldda, &info, queue);
            mic_time = magma_wtime() - mic_time;
            if (info != 0)
                printf("magma_spotrf_mic returned error %d.\n", (int) info);
            mic_perf = gflops / mic_time;

            if ( opts.check ) {
                /* =====================================================================
                   Performs operation using LAPACK
                   =================================================================== */
                cpu_time = magma_wtime();
                lapackf77_spotrf( lapack_const(opts.uplo), &N, h_A, &lda, &info );
                cpu_time = magma_wtime() - cpu_time;
                cpu_perf = gflops / cpu_time;
                if (info != 0)
                    printf("lapackf77_spotrf returned error %d.\n", (int) info);

                /* =====================================================================
                   Check the result compared to LAPACK
                   =================================================================== */
                //magma_sgetmatrix( N, N, d_A, 0, ldda, h_R, 0, lda, queue );
                /* gather matrix from mics */
                //if ( opts.uplo == MagmaUpper ) {
                    for(j = 0; j < N; j += nb) {
                        k = (j / nb) % num_mics;
                        magma_setdevice(k);
                        nk = min(nb, N - j);
                        magma_sgetmatrix( N, nk,
                                d_lA[k] + j / (nb * num_mics) * nb * ldda, 0, ldda,
                                h_R + j * lda, 0,                       lda, queue );
                    }
                /*} else {
                    for(j = 0; j < N; j += nb) {
                        k = (j / nb) % num_mics;
                        n_local = ((N / nb) / num_mics) * nb;
                        if (k < (N / nb) % num_mics)
                            n_local += nb;
                        else if (k == (N / nb) % num_mics)
                            n_local += N % nb;
                        ldn_local = ((n_local + 31) / 32) * 32;
                        ldn_local = (ldn_local % 256 == 0) ? ldn_local + 32 : ldn_local;

                        magma_setdevice(k);
                        nk = min(nb, N - j);
                        magma_sgetmatrix( nk, N,
                                d_lA[k] + j / (nb * num_mics) * nb, 0, ldn_local,
                                h_R + j, 0,                      lda, queue );
                    }
                }*/

                magma_setdevice(0);

                //error = lapackf77_slange("f", &N, &N, h_A, &lda, work);
                error = lapackf77_slansy("f", "L", &N, h_A, &lda, work);
                blasf77_saxpy(&n2, &c_neg_one, h_A, &ione, h_R, &ione);
                //error = lapackf77_slange("f", &N, &N, h_R, &lda, work) / error;
                error = lapackf77_slansy("f", "L", &N, h_R, &lda, work) / error;

                printf("%5d   %7.2f (%7.2f)   %7.2f (%7.2f)   %8.2e\n",
                        (int) N, cpu_perf, cpu_time, mic_perf, mic_time, error );
            }
            else {
                printf("%5d     ---   (  ---  )   %7.2f (%7.2f)     ---  \n",
                        (int) N, mic_perf, mic_time );
            }

            fflush(0);

            TESTING_FREE_HOST(     h_A );
            TESTING_FREE_HOST( h_R );
            for(j = 0; j < num_mics; j++) {
                magma_setdevice(j);
                TESTING_FREE_DEV( d_lA[j] );
            }
        }
        if ( opts.niter > 1 ) {
            printf( "\n" );
        }
    }

    /* Shutdown */
    magma_queue_destroy( queue );
    magma_finalize();

    return 0;
}

